# this file is for testing modules as an overlay source

# there are two ways to get make configman ignore extra symbols in modules
# first is the same way that is used in Mappings:
always_ignore_mismatches = True
# the second method is demonstrated in the values_for_module_tests_3.py file

from configman.dotdict import DotDict  # will be ignored by configman
from datetime import datetime, timedelta  # will be ignored by configman


a = 99
b = 86
c = 13  # will be ignored by configman

n = DotDict()
n.x = datetime(1960, 5, 4, 15, 10)
n.y = timedelta(1)






# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import json
from configman.def_sources import for_mappings


def setup_definitions(source, destination):
    try:
        json_dict = json.loads(source)
    except ValueError:
        with open(source) as j:
            json_dict = json.load(j)
    for_mappings.setup_definitions(json_dict, destination)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import collections
import inspect
import os

from configman.value_sources.source_exceptions import (
    NoHandlerForType,
    ModuleHandlesNothingException,
    AllHandlersFailedException,
    UnknownFileExtensionException,
    ValueException,
)
from configman.orderedset import OrderedSet
from configman.converters import str_to_python_object

from configman.config_file_future_proxy import ConfigFileFutureProxy
from configman.config_exceptions import CannotConvertError

# replace with dynamic discovery and loading
#from configman.value_sources import for_argparse
#from configman.value_sources import or_xml
from configman.value_sources import for_getopt
from configman.value_sources import for_json
from configman.value_sources import for_conf
from configman.value_sources import for_mapping
from configman.value_sources import for_configobj
from configman.value_sources import for_modules

# please replace with dynamic discovery
for_handlers = [
    for_mapping,
    for_getopt,
    for_json,
    for_conf,
    for_configobj,
    for_modules,
]


#==============================================================================
# create a dispatch table of types/objects to modules.  Each type should have
# a list of modules that can handle that type.
class DispatchByType(collections.defaultdict):
    #--------------------------------------------------------------------------
    def get_handlers(self, candidate):
        handlers_set = OrderedSet()
        # find exact candidate matches first
        for key, handler_list in self.iteritems():
            if candidate is key:
                for a_handler in handler_list:
                    handlers_set.add(a_handler)
        # then find the "instance of" candidate matches
        for key, handler_list in self.iteritems():
            if self._is_instance_of(candidate, key):
                for a_handler in handler_list:
                    handlers_set.add(a_handler)
        if not handlers_set:
            raise NoHandlerForType("no hander for %s is available" %
                                   candidate)
        return handlers_set

    #--------------------------------------------------------------------------
    @staticmethod
    def _is_instance_of(candidate, some_type):
        try:
            return isinstance(candidate, some_type)
        except TypeError:
            return False


#------------------------------------------------------------------------------
type_handler_dispatch = DispatchByType(list)
for a_handler in for_handlers:
    try:
        for a_supported_value_source in a_handler.can_handle:
            try:
                type_handler_dispatch[a_supported_value_source].append(
                    a_handler
                )
            except TypeError:
                # likely this is an instance of a handleable type that is not
                # hashable. Replace it with its base type and try to continue.
                type_handler_dispatch[type(a_supported_value_source)].append(
                    a_handler
                )
    except AttributeError:
        # this module has no can_handle attribute, therefore cannot really
        # be a handler and an error should be raised
        raise ModuleHandlesNothingException(
            "%s has no 'can_handle' attribute" % str(a_handler)
        )

file_extension_dispatch = {}
for a_handler in for_handlers:
    try:
        file_extension_dispatch[a_handler.file_name_extension] = (
            a_handler.ValueSource.write
        )
    except AttributeError:
        # this handler doesn't have a 'file_name_extension' or ValueSource
        # therefore it is not eligible for the write file dispatcher
        pass


#------------------------------------------------------------------------------
def wrap_with_value_source_api(value_source_list, a_config_manager):
    wrapped_sources = []
    for a_source in value_source_list:
        if a_source is ConfigFileFutureProxy:
            a_source = a_config_manager._get_option('admin.conf').default
            # raise hell if the config file doesn't exist
            if isinstance(a_source, basestring):
                config_file_doesnt_exist = not os.path.isfile(a_source)
                if config_file_doesnt_exist:
                    if a_config_manager.config_optional:
                        continue  # no file, it's optional, ignore it
                    raise IOError(a_source)  # no file, it's required, raise
                if a_source == a_config_manager.config_pathname:
                    # the config file has not been set to anything other than the
                    # the default value.  Force this into be the degenerate case
                    # and skip the wrapping process.  We'll read the file later.
                    continue

        if a_source is None:
            # this means the source is degenerate - like the case where
            # the config file name has not been specified
            continue
        handlers = type_handler_dispatch.get_handlers(a_source)
        wrapped_source = None
        error_history = []
        for a_handler in handlers:
            try:
                wrapped_source = a_handler.ValueSource(a_source,
                                                       a_config_manager)
                break
            except (ValueException, CannotConvertError), x:
                # a failure is not necessarily fatal, we need to try all of
                # the handlers.  It's only fatal when they've all failed
                exception_as_str = str(x)
                if exception_as_str:
                    error_history.append(str(x))
        if wrapped_source is None:
            if error_history:
                errors = '; '.join(error_history)
                raise AllHandlersFailedException(errors)
            else:
                raise NoHandlerForType(type(a_source))
        wrapped_sources.append(wrapped_source)
    return wrapped_sources


#------------------------------------------------------------------------------
def has_registration_for(config_file_type):
    return config_file_type in file_extension_dispatch


#------------------------------------------------------------------------------
def dispatch_request_to_write(config_file_type,
          options_mapping,
          opener):

    if isinstance(config_file_type, basestring):
        try:
            writer_fn = file_extension_dispatch[config_file_type]
        except KeyError:
            raise UnknownFileExtensionException(
                "%s isn't a registered file name extension" %
                config_file_type
            )
        with opener() as output_stream:
            writer_fn(options_mapping, output_stream=output_stream)
    else:
        # this is the case where we've not gotten a file extension, but a
        # for_handler module.  Use the module's ValueSource's write method
        with opener() as output_stream:
            config_file_type.ValueSource.write(
                options_mapping,
                output_stream=output_stream
            )


#------------------------------------------------------------------------------
def config_filename_from_commandline(config_manager):
    command_line_value_source = for_getopt.ValueSource(
        for_getopt.getopt,
        config_manager
    )
    values = command_line_value_source.get_values(
        config_manager,
        ignore_mismatches=True
    )
    try:
        config_file_name = values['admin.conf']
    except KeyError:
        return None

    if not os.path.isfile(config_file_name):
        # its not a file, is it a python path?
        try:
            config_object = str_to_python_object(config_file_name)
            return config_object
        except CannotConvertError:
            # ok give up, it's not a file nor a module path
            raise IOError(config_file_name)
    return config_file_name




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import datetime
import configman.datetime_util as datetime_util


#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def test_datetime_from_ISO_string(self):
        function = datetime_util.datetime_from_ISO_string

        inp = '2011-05-04T15:10:00'
        out = function(inp)
        self.assertTrue(isinstance(out, datetime.datetime))
        self.assertEqual(out.strftime('%Y-%m-%dT%H:%M:%S'), inp)

        inp = '2011-05-04'
        out = function(inp)
        self.assertTrue(isinstance(out, datetime.datetime))
        self.assertEqual(out.strftime('%Y-%m-%dT%H:%M:%S'), inp + 'T00:00:00')

        inp = '2011-05-04T15:10:00.666000'
        out = function(inp)
        self.assertTrue(isinstance(out, datetime.datetime))
        self.assertEqual(out.microsecond, 666000)
        self.assertEqual(out.strftime('%Y-%m-%dT%H:%M:%S.%f'), inp)

        # failing conditions
        self.assertRaises(ValueError, function, '211-05-04T15:10:00')
        self.assertRaises(ValueError, function, '2011-02-30T15:10:00')
        self.assertRaises(ValueError, function, '2011-02-26T24:10:00')
        self.assertRaises(ValueError, function, '2011-02-26T23:10:00.xxx')
        self.assertRaises(ValueError, function, '211-05-32')
        self.assertRaises(ValueError, function, '2011-05-32')

    #--------------------------------------------------------------------------
    def test_date_from_ISO_string(self):
        function = datetime_util.date_from_ISO_string

        inp = '2011-05-04'
        out = function(inp)
        self.assertTrue(isinstance(out, datetime.date))
        self.assertEqual(out.strftime('%Y-%m-%d'), inp)

        inp = '2011-1-2'
        out = function(inp)
        self.assertTrue(isinstance(out, datetime.date))
        self.assertEqual(out.month, 1)
        self.assertEqual(out.day, 2)

        # failing conditions
        self.assertRaises(ValueError, function, '2011-05-04T15:10:00')
        self.assertRaises(ValueError, function, '211-05-04')
        self.assertRaises(ValueError, function, '2011-05-32')

    #--------------------------------------------------------------------------
    def test_hours_str_to_timedelta(self):
        function = datetime_util.hours_str_to_timedelta
        self.assertEqual(function(1), datetime.timedelta(hours=1))

    #--------------------------------------------------------------------------
    def test_date_to_ISO_string(self):
        function = datetime_util.date_to_ISO_string
        d = datetime.datetime.now()
        self.assertEqual(
            function(d),
            d.strftime('%Y-%m-%d')
        )

        d = datetime.date.today()
        self.assertEqual(
            function(d),
            d.strftime('%Y-%m-%d')
        )

    #--------------------------------------------------------------------------
    def test_timedelta_to_seconds(self):
        function = datetime_util.timedelta_to_seconds
        self.assertEqual(function(datetime.timedelta(hours=1)), 3600)
        self.assertEqual(function(datetime.timedelta(hours=2)), 2 * 3600)
        self.assertEqual(function(datetime.timedelta(minutes=1)), 60)
        self.assertEqual(function(datetime.timedelta(seconds=1)), 1)

    #--------------------------------------------------------------------------
    def test_str_to_timedelta(self):
        function = datetime_util.str_to_timedelta
        self.assertEqual(
            function('1:1:1:01'),
            datetime.timedelta(
                days=1,
                hours=1,
                minutes=1,
                seconds=1
            )
        )
        self.assertEqual(
            function('2 00:00:00'),
            datetime.timedelta(
                days=2,
            )
        )
        self.assertEqual(
            function('01:01:01:01'),
            datetime.timedelta(
                days=1,
                hours=1,
                minutes=1,
                seconds=1
            )
        )
        self.assertEqual(
            function('1:1:1:01'),
            datetime.timedelta(
                days=1,
                hours=1,
                minutes=1,
                seconds=1
            )
        )
        self.assertEqual(
            function('1:1:1'),
            datetime.timedelta(
                hours=1,
                minutes=1,
                seconds=1
            )
        )
        self.assertEqual(
            function('1:1'),
            datetime.timedelta(
                minutes=1,
                seconds=1
            )
        )
        self.assertEqual(
            function('1'),
            datetime.timedelta(seconds=1)
        )
        self.assertRaises(ValueError, function, 'not a number')
        self.assertEqual(
            function('1'),
            datetime.timedelta(seconds=1)
        )
        self.assertRaises(TypeError, function, 10.1)



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import tempfile
import datetime

from configman import converters
from configman import RequiredConfig, Namespace, ConfigurationManager
from configman.dotdict import DotDict


#==============================================================================
# the following two classes are used in test_classes_in_namespaces_converter1
# and need to be declared at module level scope
class Foo(RequiredConfig):
    required_config = Namespace()
    required_config.add_option('x', default=17)
    required_config.add_option('y', default=23)


#==============================================================================
class Bar(RequiredConfig):
    required_config = Namespace()
    required_config.add_option('x', default=227)
    required_config.add_option('a', default=11)


# the following two classes are used in test_classes_in_namespaces_converter2
# and test_classes_in_namespaces_converter_3.  They need to be declared at
#module level scope
#==============================================================================
class Alpha(RequiredConfig):
    required_config = Namespace()
    required_config.add_option('a', doc='a', default=17)

    #--------------------------------------------------------------------------
    def __init__(self, config):
        self.config = config
        self.a = config.a

    #--------------------------------------------------------------------------
    def to_str(self):
        return "I am an instance of an Alpha object"


#==============================================================================
class AlphaBad1(Alpha):
    def __init__(self, config):
        super(AlphaBad1, self).__init__(config)
        self.a_type = int

    #--------------------------------------------------------------------------
    def to_str(self):
        raise AttributeError


#==============================================================================
class AlphaBad2(AlphaBad1):
    #--------------------------------------------------------------------------
    def to_str(self):
        raise KeyError


#==============================================================================
class AlphaBad3(AlphaBad1):
    #--------------------------------------------------------------------------
    def to_str(self):
        raise TypeError


#==============================================================================
class Beta(RequiredConfig):
    required_config = Namespace()
    required_config.add_option(
        'b',
        doc='b',
        default=23
    )

    #--------------------------------------------------------------------------
    def __init__(self, config):
        self.config = config
        self.b = config.b


#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def test_str_dict_keys(self):
        function = converters.str_dict_keys
        result = function({u'name': u'Peter', 'age': 99, 10: 11})
        self.assertEqual(result, {'name': u'Peter', 'age': 99, 10: 11})

        for key in result.keys():
            if key in ('name', 'age'):
                self.assertTrue(not isinstance(key, unicode))
                self.assertTrue(isinstance(key, str))
            else:
                self.assertTrue(isinstance(key, int))

    #--------------------------------------------------------------------------
    def test_str_quote_stripper(self):
        a = """'"single and double quoted"'"""
        self.assertEqual(
            converters.str_quote_stripper(a),
            'single and double quoted'
        )

        a = """'single quoted'"""
        self.assertEqual(converters.str_quote_stripper(a), 'single quoted')

        a = '''"double quoted"'''
        self.assertEqual(converters.str_quote_stripper(a), 'double quoted')

        a = '"""triple quoted"""'
        self.assertEqual(converters.str_quote_stripper(a), 'triple quoted')

        a = "'''triple quoted'''"
        self.assertEqual(converters.str_quote_stripper(a), 'triple quoted')

        a = '''"trailing apostrophy'"'''
        self.assertEqual(
            converters.str_quote_stripper(a),
            "trailing apostrophy'"
        )

    #--------------------------------------------------------------------------
    def test_str_to_timedelta(self):
        str_to_timedelta = converters.timedelta_converter
        from datetime import timedelta
        self.assertEqual(
            str_to_timedelta('1'),
            timedelta(seconds=1)
        )
        self.assertEqual(
            str_to_timedelta('2:1'),
            timedelta(minutes=2, seconds=1)
        )
        self.assertEqual(
            str_to_timedelta('3:2:1'),
            timedelta(hours=3, minutes=2, seconds=1)
        )
        self.assertEqual(
            str_to_timedelta('4:3:2:1'),
            timedelta(days=4, hours=3, minutes=2, seconds=1)
        )
        self.assertEqual(
            str_to_timedelta('4 03:02:01'),
            timedelta(days=4, hours=3, minutes=2, seconds=1)
        )
        self.assertRaises(ValueError, str_to_timedelta, 'xxx')
        self.assertRaises(TypeError, str_to_timedelta, 10.1)

    #--------------------------------------------------------------------------
    def test_str_to_python_object_nothing(self):
        self.assertEqual(converters.str_to_python_object(''), None)

    #--------------------------------------------------------------------------
    def test_str_to_python_object_with_whitespace(self):
        """either side whitespace doesn't matter"""
        function = converters.class_converter
        self.assertEqual(
            function("""

         configman.tests.test_converters.Foo

        """),
            Foo)

    #--------------------------------------------------------------------------
    def test_dict_conversions(self):
        d = {
            'a': 1,
            'b': 'fred',
            'c': 3.1415
        }
        converter_fn = converters.to_string_converters[type(d)]
        s = converter_fn(d)

        # round  trip
        converter_fn = converters.str_to_instance_of_type_converters[type(d)]
        dd = converter_fn(s)
        self.assertEqual(dd, d)

    #--------------------------------------------------------------------------
    def test_classes_in_namespaces_converter_1(self):
        converter_fn = converters.classes_in_namespaces_converter('HH%d')
        class_list_str = (
            'configman.tests.test_converters.Foo,'
            'configman.tests.test_converters.Bar'
        )
        result = converter_fn(class_list_str)
        self.assertTrue(hasattr(result, 'required_config'))
        req = result.required_config
        self.assertEqual(len(req), 2)
        self.assertTrue('HH0' in req)
        self.assertEqual(len(req.HH0), 1)
        self.assertTrue('cls' in req.HH0)
        self.assertTrue('HH1' in req)
        self.assertEqual(len(req.HH1), 1)
        self.assertTrue('cls' in req.HH1)
        self.assertEqual(
            sorted([x.strip() for x in class_list_str.split(',')]),
            sorted([
                x.strip() for x in
                converters.py_obj_to_str(result).split(',')
            ])
        )

    #--------------------------------------------------------------------------
    def test_classes_in_namespaces_converter_2(self):
        converter_fn = converters.classes_in_namespaces_converter('HH%d')
        class_sequence = (Foo, Bar)
        self.assertRaises(TypeError, converter_fn, class_sequence)

    #--------------------------------------------------------------------------
    def test_classes_in_namespaces_converter_3(self):
        n = Namespace()
        n.add_option(
            'kls_list',
            default=(
                'configman.tests.test_converters.Alpha, '
                'configman.tests.test_converters.Alpha, '
                'configman.tests.test_converters.Alpha'
            ),
            from_string_converter=
            converters.classes_in_namespaces_converter('kls%d')
        )

        cm = ConfigurationManager(n, argv_source=[])
        config = cm.get_config()

        self.assertEqual(len(config.kls_list.subordinate_namespace_names), 3)
        for x in config.kls_list.subordinate_namespace_names:
            self.assertTrue(x in config)
            self.assertEqual(config[x].cls, Alpha)
            self.assertTrue('cls_instance' not in config[x])

    #--------------------------------------------------------------------------
    def test_classes_in_namespaces_converter_4(self):
        n = Namespace()
        n.add_option(
            'kls_list',
            default=(
                'configman.tests.test_converters.Alpha, '
                'configman.tests.test_converters.Alpha, '
                'configman.tests.test_converters.Alpha'
            ),
            from_string_converter=converters.classes_in_namespaces_converter(
                'kls%d',
                'kls',
                instantiate_classes=True
            )
        )

        cm = ConfigurationManager(
            n,
            [{
                'kls_list': (
                    'configman.tests.test_converters.Alpha, '
                    'configman.tests.test_converters.Beta, '
                    'configman.tests.test_converters.Beta, '
                    'configman.tests.test_converters.Alpha'
                )
            }]
        )
        config = cm.get_config()

        self.assertEqual(len(config.kls_list.subordinate_namespace_names), 4)
        for x in config.kls_list.subordinate_namespace_names:
            self.assertTrue(x in config)
            self.assertTrue('kls_instance' in config[x])
            self.assertTrue(
                isinstance(config[x].kls_instance,
                           config[x].kls)
            )

    #--------------------------------------------------------------------------
    def test_to_str_to_regular_expression(self):
        import re
        self.assertEqual(
            converters.str_to_regular_expression('.*'),
            re.compile('.*')
        )

    #--------------------------------------------------------------------------
    def test_str_to_list(self):
        function = converters.list_converter
        self.assertEqual(function(''), [])

        self.assertEqual(
            function('configman.tests.test_converters.TestCase'),
            ['configman.tests.test_converters.TestCase']
        )
        self.assertEqual(
            function('configman.tests, configman'),
            ['configman.tests', 'configman']
        )
        self.assertEqual(
            function('int, str, 123, hello'),
            ['int', 'str', '123', 'hello']
        )
        self.assertEqual(
            function(u'P\xefter, L\xa3rs'),
            [u'P\xefter', u'L\xa3rs']
        )
    #--------------------------------------------------------------------------
    def test_to_str(self):
        to_str = converters.to_str
        self.assertEqual(to_str(int), 'int')
        self.assertEqual(to_str(float), 'float')
        self.assertEqual(to_str(str), 'str')
        self.assertEqual(to_str(unicode), 'unicode')
        self.assertEqual(to_str(bool), 'bool')
        self.assertEqual(to_str(dict), 'dict')
        self.assertEqual(to_str(list), 'list')
        self.assertEqual(to_str(datetime.datetime), 'datetime.datetime')
        self.assertEqual(to_str(datetime.date), 'datetime.date')
        self.assertEqual(to_str(datetime.timedelta), 'datetime.timedelta')
        self.assertEqual(to_str(type), 'type')
        self.assertEqual(
            to_str(converters.compiled_regexp_type),
            '_sre.SRE_Pattern'
        )
        self.assertEqual(to_str(1), '1')
        self.assertEqual(to_str(3.1415), '3.1415')
        self.assertEqual(
            to_str(datetime.datetime(
                1960,
                5,
                4,
                15,
                10
            )),
            '1960-05-04T15:10:00'
        )
        self.assertEqual(to_str(True), 'True')
        self.assertEqual(to_str(False), 'False')
        self.assertEqual(to_str((2, False, int, max)), '2, False, int, max')
        self.assertEqual(to_str(None), '')

    #--------------------------------------------------------------------------
    def test_str_to_boolean(self):
        self.assertTrue(converters.str_to_boolean('TRUE'))
        self.assertTrue(converters.str_to_boolean('"""TRUE"""'))
        self.assertTrue(converters.str_to_boolean('true'))
        self.assertTrue(converters.str_to_boolean('t'))
        self.assertTrue(converters.str_to_boolean('1'))
        self.assertTrue(converters.str_to_boolean('T'))
        self.assertTrue(converters.str_to_boolean('yes'))
        self.assertTrue(converters.str_to_boolean("'yes'"))
        self.assertTrue(converters.str_to_boolean('y'))

        self.assertFalse(converters.str_to_boolean('FALSE'))
        self.assertFalse(converters.str_to_boolean('false'))
        self.assertFalse(converters.str_to_boolean('f'))
        self.assertFalse(converters.str_to_boolean('F'))
        self.assertFalse(converters.str_to_boolean('no'))
        self.assertFalse(converters.str_to_boolean('NO'))
        self.assertFalse(converters.str_to_boolean(''))
        self.assertFalse(converters.str_to_boolean(
            '你好, says Lärs'
        ))
        self.assertRaises(ValueError, converters.str_to_boolean, 99)

    #--------------------------------------------------------------------------
    def test_arbitrary_object_to_string(self):
        function = converters.py_obj_to_str
        self.assertEqual(function(None), '')
        self.assertEqual(function('hello'), 'hello')

        config = DotDict()
        config.a = 17
        config.b = 23
        a = Alpha(config)
        self.assertEqual(
            converters.arbitrary_object_to_string(a),
            "I am an instance of an Alpha object"
        )
        a = AlphaBad1(config)
        self.assertEqual(
            converters.arbitrary_object_to_string(a),
            "int"
        )
        a = AlphaBad2(config)
        self.assertEqual(
            converters.arbitrary_object_to_string(a),
            "int"
        )
        a = AlphaBad3(config)
        self.assertEqual(
            converters.arbitrary_object_to_string(a),
            "int"
        )
        self.assertEqual(
            converters.arbitrary_object_to_string(IndexError),
            "IndexError"
        )

        self.assertEqual(
            converters.arbitrary_object_to_string(Beta),
            "configman.tests.test_converters.Beta"
        )

        from configman import tests as tests_module
        self.assertEqual(function(tests_module), 'configman.tests')

    #--------------------------------------------------------------------------
    def test_list_to_str(self):
        function = converters.list_to_str
        self.assertEqual(function([]), '')
        self.assertEqual(function(tuple()), '')

        import configman
        self.assertEqual(
            function([configman.tests.test_converters.TestCase]),
            'configman.tests.test_converters.TestCase'
        )
        self.assertEqual(
            function([configman.tests, configman]),
            'configman.tests, configman'
        )
        self.assertEqual(
            function([int, str, 123, "hello"]),
            'int, str, 123, hello'
        )
        self.assertEqual(
            function((configman.tests.test_converters.TestCase,)),
            'configman.tests.test_converters.TestCase'
        )
        self.assertEqual(
            function((configman.tests, configman)),
            'configman.tests, configman'
        )
        self.assertEqual(
            function((int, str, 123, "hello")),
            'int, str, 123, hello'
        )
        self.assertEqual(
            function((u'P\xefter', u'L\xa3rs')),
            u'P\xefter, L\xa3rs'
        )

    #--------------------------------------------------------------------------
    def test_to_str(self):
        self.assertEqual(converters.to_str(1), "1")
        self.assertEqual(converters.to_str(3.1415), "3.1415")
        self.assertEqual(converters.to_str('hello'), "hello")
        self.assertEqual(
            converters.to_str(u"'你好'"),
            u"'\u4f60\u597d'"
        )
        self.assertEqual(
            converters.list_to_str([1, 2, 3]),
            "1, 2, 3"
        )
        self.assertEqual(
            converters.to_str(True),
            "True"
        )
        self.assertEqual(
            converters.to_str(False),
            "False"
        )
        self.assertEqual(
            converters.to_str(datetime.datetime(1960, 5, 4, 15, 10)),
            "1960-05-04T15:10:00"
        )
        self.assertEqual(
            converters.to_str(datetime.date(1960, 5, 4)),
            "1960-05-04"
        )
        self.assertEqual(
            converters.to_str(datetime.timedelta(days=1, seconds=1)),
            "1 00:00:01"
        )
        self.assertEqual(converters.to_str(int), 'int')
        self.assertEqual(converters.to_str(float), 'float')
        self.assertEqual(converters.to_str(unittest), 'unittest')

        self.assertEqual(
            converters.to_str(converters.to_str),
            'configman.converters.to_str'
        )

        import re
        r = re.compile('.*')
        self.assertEqual(converters.to_str(r), '.*')



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import collections

from configman.converters import str_dict_keys
from configman.namespace import Namespace
from configman.option import (
    Option,
    Aggregation,
)


#------------------------------------------------------------------------------
def setup_definitions(source, destination):
    for key, val in source.items():
        if key.startswith('__'):
            continue  # ignore these
        if isinstance(val, Option):
            destination[key] = val
            if not val.name:
                val.name = key
            val.set_value(val.default)
        elif isinstance(val, Aggregation):
            destination[key] = val
        elif isinstance(val, collections.Mapping):
            if 'name' in val and 'default' in val:
                # this is an Option in the form of a dict, not a Namespace
                if key == 'not_for_definition' and val is True:
                    continue # ignore this element
                params = str_dict_keys(val)
                destination[key] = Option(**params)
            elif 'function' in val:  # this is an Aggregation
                params = str_dict_keys(val)
                destination[key] = Aggregation(**params)
            else:
                # this is a Namespace
                if key not in destination:
                    try:
                        destination[key] = Namespace(doc=val._doc)
                    except AttributeError:
                        destination[key] = Namespace()
                # recurse!
                setup_definitions(val, destination[key])
        else:
            destination[key] = Option(name=key,
                                      doc=key,
                                      default=val)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import sys
import re
import datetime
import types
import json

from configman.datetime_util import (
    datetime_from_ISO_string,
    date_from_ISO_string,
    datetime_to_ISO_string,
    date_to_ISO_string,
    timedelta_to_str,
)

# for backward compatibility these two methods get alternate names
datetime_converter = datetime_from_ISO_string
date_converter = date_from_ISO_string

from configman.config_exceptions import CannotConvertError

import datetime_util

#------------------------------------------------------------------------------
#  Utility section
#
#  various handy functions used or associated with type conversions
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def str_dict_keys(a_dict):
    """return a modified dict where all the keys that are anything but str get
    converted to str.
    E.g.

      >>> result = str_dict_keys({u'name': u'Peter', u'age': 99, 1: 2})
      >>> # can't compare whole dicts in doctests
      >>> result['name']
      u'Peter'
      >>> result['age']
      99
      >>> result[1]
      2

    The reason for this is that in Python <= 2.6.4 doing
    ``MyClass(**{u'name': u'Peter'})`` would raise a TypeError

    Note that only unicode types are converted to str types.
    The reason for that is you might have a class that looks like this::

        class Option(object):
            def __init__(self, foo=None, bar=None, **kwargs):
                ...

    And it's being used like this::

        Option(**{u'foo':1, u'bar':2, 3:4})

    Then you don't want to change that {3:4} part which becomes part of
    `**kwargs` inside the __init__ method.
    Using integers as parameter keys is a silly example but the point is that
    due to the python 2.6.4 bug only unicode keys are converted to str.
    """
    new_dict = {}
    for key in a_dict:
        if isinstance(key, unicode):
            new_dict[str(key)] = a_dict[key]
        else:
            new_dict[key] = a_dict[key]
    return new_dict


#------------------------------------------------------------------------------
def str_quote_stripper(input_str):
    if not isinstance(input_str, basestring):
        raise ValueError(input_str)
    while (
        input_str
        and input_str[0] == input_str[-1]
        and input_str[0] in ("'", '"')
    ):
        input_str = input_str.strip(input_str[0])
    return input_str


#------------------------------------------------------------------------------
#  from string section
#
#  a set of functions that will convert from a string representation into some
#  specified type
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# a bunch of known mappings of builtin items to strings
import __builtin__
known_mapping_str_to_type = dict(
    (key, val) for key, val in __builtin__.__dict__.iteritems()
    if val not in (True, False)
)

#------------------------------------------------------------------------------
from configman.datetime_util import (
    str_to_timedelta,  # don't worry about pyflakes here, unused in this file
    timedelta_to_str,  # but used elsewhere
)

timedelta_converter = str_to_timedelta  # for backward compatiblity


#------------------------------------------------------------------------------
def str_to_boolean(input_str):
    """ a conversion function for boolean
    """
    if not isinstance(input_str, basestring):
        raise ValueError(input_str)
    input_str = str_quote_stripper(input_str)
    return input_str.lower() in ("true", "t", "1", "y", "yes")

boolean_converter = str_to_boolean  # for backward compatiblity


#------------------------------------------------------------------------------
def str_to_python_object(input_str):
    """ a conversion that will import a module and class name
    """
    if not input_str:
        return None
    if not isinstance(input_str, basestring):
        # gosh, we didn't get a string, we can't convert anything but strings
        # we're going to assume that what we got is actually what was wanted
        # as the output
        return input_str
    input_str = str_quote_stripper(input_str)
    if '.' not in input_str and input_str in known_mapping_str_to_type:
        return known_mapping_str_to_type[input_str]
    parts = [x.strip() for x in input_str.split('.') if x.strip()]
    try:
        try:
            # first try as a complete module
            package = __import__(input_str)
        except ImportError:
            # it must be a class from a module
            if len(parts) == 1:
                # since it has only one part, it must be a class from __main__
                parts = ('__main__', input_str)
            package = __import__('.'.join(parts[:-1]), globals(), locals(), [])
        obj = package
        for name in parts[1:]:
            obj = getattr(obj, name)
        return obj
    except AttributeError, x:
        raise CannotConvertError("%s cannot be found" % input_str)
    except ImportError, x:
        raise CannotConvertError(str(x))

class_converter = str_to_python_object  # for backward compatibility


#------------------------------------------------------------------------------
def str_to_classes_in_namespaces(
    template_for_namespace="cls%d",
    name_of_class_option='cls',
    instantiate_classes=False
):
    """take a comma delimited  list of class names, convert each class name
    into an actual class as an option within a numbered namespace.  This
    function creates a closure over a new function.  That new function,
    in turn creates a class derived from RequiredConfig.  The inner function,
    'class_list_converter', populates the InnerClassList with a Namespace for
    each of the classes in the class list.  In addition, it puts the each class
    itself into the subordinate Namespace.  The requirement discovery mechanism
    of configman then reads the InnerClassList's requried config, pulling in
    the namespaces and associated classes within.

    For example, if we have a class list like this: "Alpha, Beta", then this
    converter will add the following Namespaces and options to the
    configuration:

        "cls0" - the subordinate Namespace for Alpha
        "cls0.cls" - the option containing the class Alpha itself
        "cls1" - the subordinate Namespace for Beta
        "cls1.cls" - the option containing the class Beta itself

    Optionally, the 'class_list_converter' inner function can embue the
    InnerClassList's subordinate namespaces with aggregates that will
    instantiate classes from the class list.  This is a convenience to the
    programmer who would otherwise have to know ahead of time what the
    namespace names were so that the classes could be instantiated within the
    context of the correct namespace.  Remember the user could completely
    change the list of classes at run time, so prediction could be difficult.

        "cls0" - the subordinate Namespace for Alpha
        "cls0.cls" - the option containing the class Alpha itself
        "cls0.cls_instance" - an instance of the class Alpha
        "cls1" - the subordinate Namespace for Beta
        "cls1.cls" - the option containing the class Beta itself
        "cls1.cls_instance" - an instance of the class Beta

    parameters:
        template_for_namespace - a template for the names of the namespaces
                                 that will contain the classes and their
                                 associated required config options.  The
                                 namespaces will be numbered sequentially.  By
                                 default, they will be "cls1", "cls2", etc.
        class_option_name - the name to be used for the class option within
                            the nested namespace.  By default, it will choose:
                            "cls1.cls", "cls2.cls", etc.
        instantiate_classes - a boolean to determine if there should be an
                              aggregator added to each namespace that
                              instantiates each class.  If True, then each
                              Namespace will contain elements for the class, as
                              well as an aggregator that will instantiate the
                              class.
                              """

    # these are only used within this method.  No need to pollute the module
    # scope with them and avoid potential circular imports
    from configman.namespace import Namespace
    from configman.required_config import RequiredConfig

    #--------------------------------------------------------------------------
    def class_list_converter(class_list_str):
        """This function becomes the actual converter used by configman to
        take a string and convert it into the nested sequence of Namespaces,
        one for each class in the list.  It does this by creating a proxy
        class stuffed with its own 'required_config' that's dynamically
        generated."""
        if isinstance(class_list_str, basestring):
            class_list = [x.strip() for x in class_list_str.split(',')]
            if class_list == ['']:
                class_list = []
        else:
            raise TypeError('must be derivative of a basestring')

        #======================================================================
        class InnerClassList(RequiredConfig):
            """This nested class is a proxy list for the classes.  It collects
            all the config requirements for the listed classes and places them
            each into their own Namespace.
            """
            # we're dynamically creating a class here.  The following block of
            # code is actually adding class level attributes to this new class
            required_config = Namespace()  # 1st requirement for configman
            subordinate_namespace_names = []  # to help the programmer know
                                              # what Namespaces we added
            namespace_template = template_for_namespace  # save the template
                                                         # for future reference
            class_option_name = name_of_class_option  # save the class's option
                                                      # name for the future
            # for each class in the class list
            for namespace_index, a_class in enumerate(class_list):
                # figure out the Namespace name
                namespace_name = template_for_namespace % namespace_index
                subordinate_namespace_names.append(namespace_name)
                # create the new Namespace
                required_config[namespace_name] = Namespace()
                # add the option for the class itself
                required_config[namespace_name].add_option(
                    name_of_class_option,
                    #doc=a_class.__doc__  # not helpful if too verbose
                    default=a_class,
                    from_string_converter=class_converter
                )
                if instantiate_classes:
                    # add an aggregator to instantiate the class
                    required_config[namespace_name].add_aggregation(
                        "%s_instance" % name_of_class_option,
                        lambda c, lc, a: lc[name_of_class_option](lc)
                    )

            @classmethod
            def to_str(cls):
                """this method takes this inner class object and turns it back
                into the original string of classnames.  This is used
                primarily as for the output of the 'help' option"""
                return ', '.join(
                    py_obj_to_str(v[name_of_class_option].value)
                    for v in cls.get_required_config().values()
                    if isinstance(v, Namespace)
                )

        return InnerClassList  # result of class_list_converter
    return class_list_converter  # result of classes_in_namespaces_converter

# for backward compatibility
classes_in_namespaces_converter = str_to_classes_in_namespaces


#------------------------------------------------------------------------------
def str_to_regular_expression(input_str):
    return re.compile(input_str)

regex_converter = str_to_regular_expression  # for backward compatibility

compiled_regexp_type = type(re.compile(r'x'))


#------------------------------------------------------------------------------
def str_to_list(
    input_str,
    item_converter=lambda x: x,
    item_separator=',',
    list_to_collection_converter=None,
):
    """ a conversion function for list
    """
    if not isinstance(input_str, basestring):
        raise ValueError(input_str)
    input_str = str_quote_stripper(input_str)
    result = [
        item_converter(x.strip())
        for x in input_str.split(item_separator) if x.strip()
    ]
    if list_to_collection_converter is not None:
        return list_to_collection_converter(result)
    return result

list_converter = str_to_list  # for backward compatibility


#------------------------------------------------------------------------------
#
#   To string section
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
str_to_instance_of_type_converters = {
    int: int,
    float: float,
    str: str,
    unicode: unicode,
    bool: boolean_converter,
    dict: json.loads,
    list: list_converter,
    datetime.datetime: datetime_converter,
    datetime.date: date_converter,
    datetime.timedelta: timedelta_converter,
    type: class_converter,
    types.FunctionType: class_converter,
    compiled_regexp_type: regex_converter,
}

 # backward compatibility
from_string_converters = str_to_instance_of_type_converters


#------------------------------------------------------------------------------
def arbitrary_object_to_string(a_thing):
    """take a python object of some sort, and convert it into a human readable
    string.  this function is used extensively to convert things like "subject"
    into "subject_key, function -> function_key, etc."""
    # is it None?
    if a_thing is None:
        return ''
    # is it already a string?
    if isinstance(a_thing, basestring):
        return a_thing
    # does it have a to_str function?
    try:
        return a_thing.to_str()
    except (AttributeError, KeyError, TypeError):
        # AttributeError - no to_str function?
        # KeyError - DotDict has no to_str?
        # TypeError - problem converting
        # nope, no to_str function
        pass
    # is this a type proxy?
    try:
        return arbitrary_object_to_string(a_thing.a_type)
    except (AttributeError, KeyError, TypeError):
        #
        # nope, no a_type property
        pass
    # is it a built in?
    try:
        return known_mapping_type_to_str[a_thing]
    except (KeyError, TypeError):
        # nope, not a builtin
        pass
    # is it something from a loaded module?
    try:
        if a_thing.__module__ not in ('__builtin__', 'exceptions'):
            if a_thing.__module__ == "__main__":
                module_name = (
                    sys.modules['__main__']
                    .__file__[:-3]
                    .replace('/', '.')
                    .strip('.')
                )
            else:
                module_name = a_thing.__module__
            return "%s.%s" % (module_name, a_thing.__name__)
    except AttributeError:
        # nope, not one of these
        pass
    # maybe it has a __name__ attribute?
    try:
        return a_thing.__name__
    except AttributeError:
        # nope, not one of these
        pass
    # punt and see what happens if we just cast it to string
    return str(a_thing)


py_obj_to_str = arbitrary_object_to_string  # for backwards compatibility


#------------------------------------------------------------------------------
def list_to_str(a_list, delimiter=', '):
    return delimiter.join(to_str(x) for x in a_list)

#------------------------------------------------------------------------------
known_mapping_type_to_str = dict(
    (val, key) for key, val in __builtin__.__dict__.iteritems()
    if val not in (True, False, list, dict)
)

#------------------------------------------------------------------------------
to_string_converters = {
    int: str,
    float: str,
    str: str,
    unicode: unicode,
    list: list_to_str,
    tuple: list_to_str,
    bool: lambda x: 'True' if x else 'False',
    dict: json.dumps,
    datetime.datetime: datetime_to_ISO_string,
    datetime.date: date_to_ISO_string,
    datetime.timedelta: timedelta_to_str,
    type: arbitrary_object_to_string,
    types.ModuleType: arbitrary_object_to_string,
    types.FunctionType: arbitrary_object_to_string,
    compiled_regexp_type: lambda x: x.pattern,
}


#------------------------------------------------------------------------------
def to_str(a_thing):
    try:
        return to_string_converters[type(a_thing)](a_thing)
    except KeyError:
        return arbitrary_object_to_string(a_thing)

#------------------------------------------------------------------------------
converters_requiring_quotes = [eval, regex_converter]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import sys
import re
import os.path

import configobj

from configman.value_sources.source_exceptions import (
    CantHandleTypeException,
    ValueException,
    NotEnoughInformationException
)
from configman.namespace import Namespace
from configman.option import Option

from configman.dotdict import DotDict
from configman.memoize import memoize

file_name_extension = 'ini'

can_handle = (
    configobj,
    configobj.ConfigObj,
    basestring,
)


#==============================================================================
class ConfigObjWithIncludes(configobj.ConfigObj):
    """This derived class is an extention to ConfigObj that adds nested
    includes to ini files.  Here's an example:

    db.ini:

        dbhostname=myserver
        dbname=some_database
        dbuser=dwight
        dbpassword=secrets

    app.ini:
        [source]
        +include ./db.ini

        [destination]
        +include ./db.ini

    when the 'app.ini' file is loaded, ConfigObj will respond as if the file
    had been written like this:
        [source]
        dbhostname=myserver
        dbname=some_database
        dbuser=dwight
        dbpassword=secrets

        [destination]
        dbhostname=myserver
        dbname=some_database
        dbuser=dwight
        dbpassword=secrets
    """
    _include_re = re.compile(r'^(\s*)\+include\s+(.*?)\s*$')

    #--------------------------------------------------------------------------
    def _expand_files(self, file_name, original_path, indent=""):
        """This recursive function accepts a file name, opens the file and then
        spools the contents of the file into a list, examining each line as it
        does so.  If it detects a line beginning with "+include", it assumes
        the string immediately following is a file name.  Recursing, the file
        new file is openned and its contents are spooled into the accumulating
        list."""
        expanded_file_contents = []
        with open(file_name) as f:
            for a_line in f:
                match = ConfigObjWithIncludes._include_re.match(a_line)
                if match:
                    include_file = match.group(2)
                    include_file = os.path.join(
                        original_path,
                        include_file
                    )
                    new_lines = self._expand_files(
                        include_file,
                        os.path.dirname(include_file),
                        indent + match.group(1)
                    )
                    expanded_file_contents.extend(new_lines)
                else:
                    expanded_file_contents.append(indent + a_line.rstrip())
        return expanded_file_contents

    #--------------------------------------------------------------------------
    def _load(self, infile, configspec):
        """this overrides the original ConfigObj method of the same name.  It
        runs through the input file collecting lines into a list.  When
        completed, this method submits the list of lines to the super class'
        function of the same name.  ConfigObj proceeds, completely unaware
        that it's input file has been preprocessed."""
        if isinstance(infile, basestring):
            original_path = os.path.dirname(infile)
            expanded_file_contents = self._expand_files(infile, original_path)
            super(ConfigObjWithIncludes, self)._load(
                expanded_file_contents,
                configspec
            )
        else:
            super(ConfigObjWithIncludes, self)._load(infile, configspec)


#==============================================================================
class LoadingIniFileFailsException(ValueException):
    pass


#==============================================================================
class ValueSource(object):

    #--------------------------------------------------------------------------
    def __init__(
        self, source,
        config_manager=None,
        top_level_section_name=''
    ):
        self.delayed_parser_instantiation = False
        self.top_level_section_name = top_level_section_name
        if source is configobj.ConfigObj:
            try:
                app = config_manager._get_option('admin.application')
                source = "%s.%s" % (app.value.app_name, file_name_extension)
            except AttributeError:
                # we likely don't have the admin.application object set up yet.
                # we need to delay the instantiation of the config parser
                # until later.
                if source is None:
                    raise NotEnoughInformationException(
                        "Can't setup an ini file without knowing the file name"
                    )
                self.delayed_parser_instantiation = True
                return
            if not os.path.exists(source) and config_manager.config_optional:
                return
        if (
            isinstance(source, basestring) and
            source.endswith(file_name_extension)
        ):
            try:
                self.config_obj = ConfigObjWithIncludes(source)
            except Exception, x:
                raise LoadingIniFileFailsException(
                    "ConfigObj cannot load ini: %s" % str(x)
                )
        else:
            raise CantHandleTypeException()

    #--------------------------------------------------------------------------
    @memoize()
    def get_values(self, config_manager, ignore_mismatches, obj_hook=DotDict):
        """Return a nested dictionary representing the values in the ini file.
        In the case of this ValueSource implementation, both parameters are
        dummies."""
        if self.delayed_parser_instantiation:
            try:
                app = config_manager._get_option('admin.application')
                source = "%s%s" % (app.value.app_name, file_name_extension)
                self.config_obj = configobj.ConfigObj(source)
                self.delayed_parser_instantiation = False
            except AttributeError:
                # we don't have enough information to get the ini file
                # yet.  we'll ignore the error for now
                return obj_hook()  # return empty dict of the obj_hook type
        if isinstance(self.config_obj, obj_hook):
            return self.config_obj
        return obj_hook(initializer=self.config_obj)

    #--------------------------------------------------------------------------
    @staticmethod
    def write(source_mapping, output_stream=sys.stdout):
        ValueSource._write_ini(source_mapping, output_stream=output_stream)

    #--------------------------------------------------------------------------
    @staticmethod
    def _namespace_reference_value_from_sort(key_value_tuple):
        key, value = key_value_tuple
        if value._reference_value_from:
            return 'aaaaaa' + key
        else:
            return key

    #--------------------------------------------------------------------------
    @staticmethod
    def _write_ini(source_dict, namespace_name=None, level=0, indent_size=4,
                   output_stream=sys.stdout):
        """this function prints the components of a configobj ini file.  It is
        recursive for outputing the nested sections of the ini file."""
        options = [
            value
            for value in source_dict.values()
            if isinstance(value, Option)
        ]
        options.sort(cmp=lambda x, y: cmp(x.name, y.name))
        indent_spacer = " " * (level * indent_size)
        for an_option in options:
            print >>output_stream, "%s# %s" % (indent_spacer, an_option.doc)
            option_value = str(an_option)
            if isinstance(option_value, unicode):
                option_value = option_value.encode('utf8')

            if an_option.reference_value_from:
                print >>output_stream, (
                    '%s# see "%s.%s" for the default or override it here' % (
                        indent_spacer,
                        an_option.reference_value_from,
                        an_option.name
                    )
                )

            if an_option.likely_to_be_changed or an_option.has_changed:
                option_format = '%s%s=%s\n'
            else:
                option_format = '%s#%s=%s\n'

            if isinstance(option_value, basestring) and ',' in option_value:
                # quote lists unless they're already quoted
                if option_value[0] not in '\'"':
                    option_value = '"%s"' % option_value

            print >>output_stream, option_format % (
                indent_spacer,
                an_option.name,
                option_value
            )
        next_level = level + 1
        namespaces = [
            (key, value)
            for key, value in source_dict.items()
            if isinstance(value, Namespace)
        ]
        namespaces.sort(key=ValueSource._namespace_reference_value_from_sort)
        for key, namespace in namespaces:
            next_level_spacer = " " * next_level * indent_size
            print >>output_stream, "%s%s%s%s\n" % (
                indent_spacer,
                "[" * next_level,
                key,
                "]" * next_level
            )
            if namespace._doc:
                print >>output_stream, (
                    "%s%s" % (next_level_spacer, namespace._doc)
                )
            if namespace._reference_value_from:
                print >>output_stream, (
                    "%s#+include ./common_%s.ini\n"
                    % (next_level_spacer, key)
                )

            if namespace_name:
                ValueSource._write_ini(
                    source_dict=namespace,
                    namespace_name="%s.%s" % (namespace_name, key),
                    level=level+1,
                    indent_size=indent_size,
                    output_stream=output_stream
                )
            else:
                ValueSource._write_ini(
                    source_dict=namespace,
                    namespace_name=key,
                    level=level+1,
                    indent_size=indent_size,
                    output_stream=output_stream
                )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from configman import config_exceptions


class ValueException(config_exceptions.ConfigmanException):
    pass


class NotEnoughInformationException(ValueException):
    pass


class LoadingIniFileFailsException(ValueException):
    pass


class UnknownFileExtensionException(ValueException):
    pass


class ModuleHandlesNothingException(ValueException):
    pass


class NoHandlerForType(ValueException):
    pass


class AllHandlersFailedException(ValueException):
    pass


class CantHandleTypeException(ValueException):
    pass


class UnknownFileExtension(ValueException):
    pass



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import os
import tempfile
import contextlib
from cStringIO import StringIO

from configman.datetime_util import datetime_from_ISO_string

from configman.value_sources import for_conf
from configman.namespace import Namespace
from configman.config_manager import ConfigurationManager
from configman.dotdict import DotDict, DotDictWithAcquisition


def stringIO_context_wrapper(a_stringIO_instance):
    @contextlib.contextmanager
    def stringIS_context_manager():
        yield a_stringIO_instance
    return stringIS_context_manager


#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def _some_namespaces(self):
        """set up some namespaces"""
        n = Namespace(doc='top')
        n.add_option(
            'aaa',
            '2011-05-04T15:10:00',
            'the a',
            short_form='a',
            from_string_converter=datetime_from_ISO_string
        )
        n.c = Namespace(doc='c space')
        n.c.add_option(
            'fred',
            'stupid',
            # deliberate whitespace to test that it gets stripped
            ' husband from Flintstones '
        )
        n.c.add_option('wilma', 'waspish', 'wife from Flintstones')
        n.c.e = Namespace(doc='e space')
        n.c.e.add_option(
            'dwight',
            default=97,
            doc='my uncle'
        )
        n.c.add_option(
            'dwight',
            default=98,
            doc='your uncle'
        )
        n.d = Namespace(doc='d space')
        n.d.add_option('fred', 'crabby', 'male neighbor from I Love Lucy')
        n.d.add_option(
            'ethel',
            'silly',
            'female neighbor from I Love Lucy'
        )
        n.x = Namespace(doc='x space')
        n.x.add_option('size', 100, 'how big in tons', short_form='s')
        n.x.add_option('password', 'secret', 'the password')
        return n

    #--------------------------------------------------------------------------
    def test_for_conf_basics(self):
        tmp_filename = os.path.join(tempfile.gettempdir(), 'test.conf')
        with open(tmp_filename, 'w') as f:
            f.write('# comment\n')
            f.write('limit=20\n')
            f.write('\n')
        try:
            o = for_conf.ValueSource(tmp_filename)
            assert o.values == {'limit': '20'}, o.values
            # in the case of this implementation of a ValueSource,
            # the two parameters to get_values are dummies.  That may
            # not be true for all ValueSource implementations
            self.assertEqual(o.get_values(1, False), {'limit': '20'})
            self.assertEqual(o.get_values(2, True), {'limit': '20'})

            v = o.get_values(None, True, DotDict)
            self.assertTrue(isinstance(v, DotDict))
            v = o.get_values(None, None, obj_hook=DotDictWithAcquisition)
            self.assertTrue(isinstance(v, DotDictWithAcquisition))
        finally:
            if os.path.isfile(tmp_filename):
                os.remove(tmp_filename)

    #--------------------------------------------------------------------------
    def donttest_for_conf_nested_namespaces(self):
        n = self._some_namespaces()
        cm = ConfigurationManager(
            n,
            values_source_list=[],
        )
        out = StringIO()
        cm.write_conf(for_conf, opener=stringIO_context_wrapper(out))
        received = out.getvalue()
        out.close()
        expected = """# name: aaa
# doc: the a
aaa=2011-05-04T15:10:00

#------------------------------------------------------------------------------
# c - c space

# name: c.dwight
# doc: your uncle
c.dwight=98

# name: c.fred
# doc: husband from Flintstones
c.fred=stupid

# name: c.wilma
# doc: wife from Flintstones
c.wilma=waspish

#------------------------------------------------------------------------------
# e - e space

# name: c.e.dwight
# doc: my uncle
c.e.dwight=97

#------------------------------------------------------------------------------
# d - d space

# name: d.ethel
# doc: female neighbor from I Love Lucy
d.ethel=silly

# name: d.fred
# doc: male neighbor from I Love Lucy
d.fred=crabby

#------------------------------------------------------------------------------
# x - x space

# name: x.password
# doc: the password
x.password=secret

# name: x.size
# doc: how big in tons
x.size=100"""
        self.assertEqual(received.strip(), expected)

        strio = StringIO(expected)
        n.c.dwight.default = 3823
        n.c.e.dwight = 'fred'
        cm2 = ConfigurationManager(
            n,
            [stringIO_context_wrapper(strio)],
            use_admin_controls=False,
            use_auto_help=False
        )
        result = cm2.get_config()
        self.assertEqual(len(result), 4)
        self.assertEqual(sorted(result.keys()), ['aaa', 'c', 'd', 'x'])
        self.assertEqual(len(result.c), 4)
        self.assertEqual(
            sorted(result.c.keys()),
            ['dwight', 'e', 'fred', 'wilma']
        )
        self.assertEqual(result.c.dwight, 98)
        self.assertEqual(len(result.c.e), 1)
        self.assertEqual(result.c.e.dwight, '97')



# this file is for testing modules as an overlay source

# none of the symbols in this file have been set to be ignored.  That means
# that configman will warn that there are extra symbols in this file that it
# doesn't know about.  Further, if '--admin.strict' is enabled, the extra
# symbols are a fatal error

from configman.dotdict import DotDict
from configman import RequiredConfig
from datetime import timedelta, date

b = 'now is the time'

n = DotDict()
n.y = timedelta(1)
n.z = date(1960, 5, 4)


class Alpha(RequiredConfig):
    required_config = {
        'host': 'localhost',
        'port':  5432,
    }

dynamic_load = Alpha


# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import types
import sys
import datetime
import json
import keyword
import re

from inspect import isclass, ismodule, isfunction
from types import NoneType
from collections import defaultdict

from configman.namespace import Namespace
from configman.dotdict import DotDict
from configman.option import Option, Aggregation
from configman.converters import (
    to_str,
    class_converter,
    known_mapping_str_to_type,
    CannotConvertError,
    str_quote_stripper,
    compiled_regexp_type
)
file_name_extension = 'py'


can_handle = (
    types.ModuleType,
    basestring
)

#------------------------------------------------------------------------------
# Converter Section
#------------------------------------------------------------------------------

# each value source is allowed to have its own set of converters for
# serializing objects to strings.  This converter section defines the functions
# that can serialize objects into Python code.  This allows writing of a
# Python module in the same manner that ini files are written.

identifier_re = re.compile(r'^[a-z_][a-z0-9_\.]*$', re.I)


#------------------------------------------------------------------------------
def is_identifier(a_candidate):
    if a_candidate in keyword.kwlist:
        return False  # identifiers can't also be keywords
    return bool(identifier_re.match(a_candidate))

#------------------------------------------------------------------------------
def sequence_to_string(
    a_list,
    open_bracket_char='[',
    close_bracket_char=']',
    delimiter=", "
):
    """a dedicated function that turns a list into a comma delimited string
    of items converted.  This method will flatten nested lists."""
    return "%s%s%s" % (
        open_bracket_char,
        delimiter.join(
            local_to_str(x)
            for x in a_list
        ),
        close_bracket_char
    )


#------------------------------------------------------------------------------
def dict_to_string(d):
    return json.dumps(
        d,
        indent=4,
        sort_keys=True,
        separators=(',', ': '),
    )


#------------------------------------------------------------------------------
def string_to_string(a_string):
    quote = '"'
    if '"' in a_string:
        quote = "'"
    if "'" in a_string:
        quote = '"""'
    if "/n" in a_string:
        quote = "'''"
    return "%s%s%s" % (quote, a_string, quote)


#------------------------------------------------------------------------------
def unicode_to_unicode(a_string):
    quote = '"'
    if '"' in a_string:
        quote = "'"
    if "'" in a_string:
        quote = '"""'
    if "/n" in a_string:
        quote = "'''"
    return "u%s%s%s" % (quote, a_string, quote)


#------------------------------------------------------------------------------
def datetime_to_string(d):
    return "datetime(year=%s, month=%s, day=%s, hour=%s, " \
        "minute=%s, second=%s)" % (
            d.year,
            d.month,
            d.day,
            d.hour,
            d.minute,
            d.second,
        )


#------------------------------------------------------------------------------
def date_to_string(d):
    return "date(year=%s, month=%s, day=%s)" % (
        d.year,
        d.month,
        d.day,
    )


#------------------------------------------------------------------------------
def timedelta_to_string(d):
    return "timedelta(days=%s, seconds=%s)" % (
        d.days,
        d.seconds,
    )


#------------------------------------------------------------------------------
def get_import_for_type(t):
    """given a type, return a tuple of the (module-path, type_name)
    or (None, None) if it is a built in."""
    t_as_string = to_str(t)
    if not is_identifier(t_as_string):
        # this class expanded into something other than a single identifier
        # we can ignore it.  This is the case when we encounter something
        # like the configman.converter.str_to_classes_in_namespaces
        # InnerClassList.  We can safely ignore these things here.
        return (None, None)
    if '.' in t_as_string:
        parts = t_as_string.split('.')
        return ('.'.join(parts[:-1]), parts[-1])
    else:
        if t_as_string in known_mapping_str_to_type:
            return (None, None)
        return (None, t_as_string)


#------------------------------------------------------------------------------
local_to_string_converters = {
    str: string_to_string,
    unicode: unicode_to_unicode,
    list: sequence_to_string,
    tuple: sequence_to_string,
    dict: dict_to_string,
    datetime.datetime: datetime_to_string,
    datetime.date: date_to_string,
    datetime.timedelta: timedelta_to_string,
    NoneType: lambda x: "None",
    compiled_regexp_type: lambda x: string_to_string(x.pattern)
}


#------------------------------------------------------------------------------
def find_to_string_converter(a_thing):
    for a_candidate_type, to_string_converter in local_to_string_converters:
        if isinstance(a_thing, a_candidate_type):
            return to_string_converter
    return None


#------------------------------------------------------------------------------
def local_to_str(a_thing):
    try:
        return local_to_string_converters[type(a_thing)](a_thing)
    except KeyError:
        try:
            return find_to_string_converter(a_thing)(a_thing)
        except TypeError:
            return to_str(a_thing)


#==============================================================================
class ValueSource(object):
    #--------------------------------------------------------------------------
    def __init__(self, source, the_config_manager=None):
        if isinstance(source, basestring):
            source = class_converter(source)
        module_as_dotdict = DotDict()
        try:
            ignore_symbol_list = source.ignore_symbol_list
            if 'ignore_symbol_list' not in ignore_symbol_list:
                ignore_symbol_list.append('ignore_symbol_list')
        except AttributeError:
            ignore_symbol_list = []
        try:
            self.always_ignore_mismatches = source.always_ignore_mismatches
        except AttributeError:
            pass  # don't need to do anything - mismatches will not be ignored
        for key, value in source.__dict__.iteritems():
            if key.startswith('__') and key != "__doc__":
                continue
            if key in ignore_symbol_list:
                continue
            module_as_dotdict[key] = value
        self.module = source
        self.source = module_as_dotdict

    #--------------------------------------------------------------------------
    def get_values(self, config_manager, ignore_mismatches, obj_hook=DotDict):
        if isinstance(self.source, obj_hook):
            return self.source
        return obj_hook(initializer=self.source)

    #--------------------------------------------------------------------------
    @staticmethod
    def write_class(key, value, alias_by_class, output_stream):
        if value in alias_by_class:
            class_str = alias_by_class[value]
        else:
            class_str = local_to_str(value)
        if is_identifier(class_str):
            parts = [x.strip() for x in class_str.split('.') if x.strip()]
            print >>output_stream, '%s = %s' % (key, parts[-1])
        else:
            print >>output_stream, '%s = "%s"' % (key, class_str)

    #--------------------------------------------------------------------------
    @staticmethod
    def write_bare_value(key, value, output_stream):
        if isclass(value):
            ValueSource.write_class(key, value, output_stream)
            return
        try:
            value = local_to_str(value)
        except CannotConvertError:
            value = repr(value)
        if '\n' in value:
            value = "'''%s'''" % str_quote_stripper(value)
        print >>output_stream, '%s = %s' % (key, value)

    #--------------------------------------------------------------------------
    @staticmethod
    def write_option(key, an_option, alias_by_class, output_stream):
        print >>output_stream, '\n',
        if an_option.doc:
            print >>output_stream, '# %s' % an_option.doc
        if (
            isclass(an_option.value)
            or ismodule(an_option.value)
            or isfunction(an_option.value)
        ):
            ValueSource.write_class(
                key,
                an_option.value,
                alias_by_class,
                output_stream
            )
            return
        else:
            value = local_to_str(an_option.value)
            print >>output_stream, '%s = %s' % (key, value)

    #--------------------------------------------------------------------------
    @staticmethod
    def write_namespace(key, a_namespace, output_stream):
        print >>output_stream, '\n# Namespace:', key
        if hasattr(a_namespace, 'doc'):
            print >>output_stream, '#', a_namespace.doc
        print >>output_stream, '%s = DotDict()' % key

    #--------------------------------------------------------------------------
    @staticmethod
    def write(source_mapping, output_stream=sys.stdout):
        """This method writes a Python module respresenting all the keys
        and values known to configman.
        """
        # a set of classes, modules and/or functions that are values in
        # configman options.  These values will have to be imported in the
        # module that this method is writing.
        set_of_classes_needing_imports = set()
        # once symbols are imported, they are in the namespace of the module,
        # but that's not where we want them.  We only want them to be values
        # in configman Options.  This set will be used to make a list of
        # these symbols, to forewarn a future configman that reads this
        # module, that it can ignore these symbols. This will prevent that
        # future configman from issuing a "mismatced symbols" warinng.
        symbols_to_ignore = set()

        # look ahead to see what sort of imports we're going to have to do
        for key in source_mapping.keys_breadth_first():
            value = source_mapping[key]

            if isinstance(value, Aggregation):
                # Aggregations don't get included, skip on
                continue

            if '.' in key:
                # this indicates that there are things in nested namespaces,
                # we will use the DotDict class to represent namespaces
                set_of_classes_needing_imports.add(DotDict)

            option = None
            if isinstance(value, Option):
                # it's the value inside the option, not the option itself
                # that is of interest to us
                option = value
                value = option.value

            if value is None:
                # we don't need in import anything having to do with None
                continue

            if isclass(value) or ismodule(value) or isfunction(value):
                # we know we need to import any of these types
                set_of_classes_needing_imports.add(value)
            else:
                try:
                    # perhaps the value is an instance of a class?  If so,
                    # we'll likely need to import that class, but only if
                    # we don't have a way to convert a string to that class
                    set_of_classes_needing_imports.add(value.__class__)
                except AttributeError:
                    # it's not a class instance, we can skip on
                    pass

        # for everyone of the imports that we're going to have to create
        # we need to know the dotted module pathname and the name of the
        # of the class/module/function.  This routine make a list of 3-tuples
        # class, dotted_module_path, class_name
        class_and_module_path_and_class_name = []
        for a_class in set_of_classes_needing_imports:
            module_path, class_name = get_import_for_type(a_class)
            if (not module_path) and (not class_name):
                continue
            class_and_module_path_and_class_name.append(
                (a_class, module_path, class_name)
            )

        # using the collection of 3-tuples, create a lookup mapping where a
        # class is the key to a 2-tuple of the dotted_module_path & class_name.
        # This is also the appropriate time to detect any name collisions
        # and create a mapping of aliases, so we can resolve name collisions.
        class_name_by_module_path_list = defaultdict(list)
        alias_by_class = {}
        previously_used_names = set()
        for (
            a_class,
            a_module_path,
            class_name
        ) in class_and_module_path_and_class_name:
            if class_name:
                if class_name in previously_used_names:
                    new_class_name_alias = "%s_%s" % (
                        a_module_path.replace('.', '_'),
                        class_name
                    )
                    alias_by_class[a_class] = new_class_name_alias
                    previously_used_names.add(new_class_name_alias)
                else:
                    previously_used_names.add(class_name)
                class_name_by_module_path_list[a_module_path].append(
                    (a_class, class_name)
                )

        # start writing the output module
        print >>output_stream, "# generated Python configman file\n"

        # the first section that we're going to write is imports of the form:
        #     from X import Y
        # and
        #     from X import (
        #         A,
        #         B,
        #     )
        for a_module_path in sorted(class_name_by_module_path_list.keys()):
            # if there is no module path, then it is something that we don't
            # need to import.  If the module path begins with underscore then
            # it is private and we ought not step into that mire.  If that
            # causes the output module to fail, it is up to the implementer
            # of the configman option to have created an approprate
            # "from_string" & "to_string" configman Option function references.
            if a_module_path is None or a_module_path.startswith('_'):
                continue
            list_of_class_names = \
                class_name_by_module_path_list[a_module_path]
            if len(list_of_class_names) > 1:
                output_line = "from %s import (\n" % a_module_path
                for a_class, a_class_name in sorted(list_of_class_names):
                    if a_class in alias_by_class:
                        output_line =  "%s\n    %s as %s," % (
                            output_line,
                            a_class_name,
                            alias_by_class[a_class]
                        )
                        symbols_to_ignore.add(alias_by_class[a_class])
                    else:
                        output_line = "%s    %s,\n" % (
                            output_line,
                            a_class_name
                        )
                        symbols_to_ignore.add(a_class_name)

                output_line = output_line + ')'
                print >>output_stream, output_line.strip()
            else:
                a_class, a_class_name = list_of_class_names[0]
                output_line = "from %s import %s" % (
                    a_module_path,
                    a_class_name
                )
                if a_class in alias_by_class:
                    output_line = "%s as %s" % (
                        output_line,
                        alias_by_class[a_class]
                    )
                    symbols_to_ignore.add(alias_by_class[a_class])
                else:
                    symbols_to_ignore.add(a_class_name)
                print >>output_stream, output_line.strip()
        print >>output_stream, ''

        # The next section to write will be the imports of the form:
        #     import X
        for a_module_path in sorted(class_name_by_module_path_list.keys()):
            list_of_class_names = \
                class_name_by_module_path_list[a_module_path]
            a_class, a_class_name = list_of_class_names[0]
            if a_module_path:
                continue
            import_str = ("import %s" % a_class_name).strip()
            symbols_to_ignore.add(a_class_name)
            print  >>output_stream, import_str

        # See the explanation of 'symbols_to_ignore' above
        if symbols_to_ignore:
            print >>output_stream, "\n" \
                "# the following symbols will be ignored by configman when\n" \
                "# this module is used as a value source.  This will\n" \
                "# suppress the mismatch warning since these symbols are\n" \
                "# values for options, not option names themselves."
            print >>output_stream, "ignore_symbol_list = ["
            for a_symbol in symbols_to_ignore:
                print >>output_stream, '    "%s",' % a_symbol
            print >>output_stream, ']\n'

        # finally, as the last step, we need to write out the keys and values
        # will be used by a future configman as Options and values.
        sorted_keys = sorted(
            source_mapping.keys_breadth_first(include_dicts=True)
        )
        for key in sorted_keys:
            value = source_mapping[key]
            if isinstance(value, Namespace):
                ValueSource.write_namespace(key, value, output_stream)
            elif isinstance(value, Option):
                ValueSource.write_option(
                    key,
                    value,
                    alias_by_class,
                    output_stream
                )
            elif isinstance(value, Aggregation):
                # skip Aggregations
                continue
            else:
                ValueSource.write_bare_value(key, value, output_stream)



"""This is a test module for the configman.value_sources.for_modules
component. It is used only with the configman.tests.test_val_for_modules tests
"""
# WARNING - do not change the first four words of the doc string above.  Those
# words are used in a test in the file configman.tests.test_val_for_modules

import os
import collections

a = 18

b = [1, 2, 3]
b.append(3)

c = set(b)

d = {
    'a': a,
    'b': b,
    'c': c,
    'd': {
        1: 'one',
        2: 'two',
    }
}


def foo(a=None, b=None, c=None):
    return "%s%s%s" % (a, b, c)

from functools import partial
bar = partial(foo, a=a)

class Alpha(object):
    def __init__(self, a=None, b=None, c=None):
        self.a = a
        self.b = b
        self.c = c

    def __str__(self):
        return foo(self.a, self.b, self.c)



x = 23
y = 'this is private'

ignore_symbol_list = ['x', 'y', 'os', 'partial']


# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# this will be expanded in the future when additional command line libraries
# are supported

# at which point we want to make argparse the default, we will eliminate
# this line
import getopt as command_line



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import os
import tempfile
from cStringIO import StringIO
import contextlib

from configman.datetime_util import (
    datetime_from_ISO_string
)
from configman.namespace import Namespace
from configman.config_manager import ConfigurationManager
from configman.config_exceptions import NotAnOptionError
from configman.dotdict import DotDict, DotDictWithAcquisition

try:
    #from ..value_sources.for_configobj import ValueSource
    from configman.value_sources import for_configobj
except ImportError:
    # this module is optional.  If it doesn't exsit, that's ok, we'll just
    # igrore the tests
    pass
else:

    #--------------------------------------------------------------------------
    def stringIO_context_wrapper(a_stringIO_instance):
        @contextlib.contextmanager
        def stringIS_context_manager():
            yield a_stringIO_instance
        return stringIS_context_manager

    #==========================================================================
    class TestCase(unittest.TestCase):
        def _some_namespaces(self):
            """set up some namespaces"""
            n = Namespace(doc='top')
            n.add_option(
                'aaa',
                '2011-05-04T15:10:00',
                'the a',
                short_form='a',
                from_string_converter=datetime_from_ISO_string
            )
            n.c = Namespace(doc='c space')
            n.c.add_option(
                'fred',
                'stupid, deadly',
                # deliberate whitespace to test that it gets stripped
                ' husband from Flintstones '
            )
            n.c.add_option('wilma', "waspish's", 'wife from Flintstones')
            n.d = Namespace(doc='d space')
            n.d.add_option('fred', "crabby", 'male neighbor from I Love Lucy')
            n.d.add_option(
                'ethel',
                'silly',
                'female neighbor from I Love Lucy'
            )
            n.x = Namespace(doc='x space')
            n.x.add_option('size', 100, 'how big in tons', short_form='s')
            n.x.add_option('password', 'secret "message"', 'the password')
            return n

        #----------------------------------------------------------------------
        def test_for_configobj_basics(self):
            """test basic use of for_configobj"""
            tmp_filename = os.path.join(tempfile.gettempdir(), 'test.ini')
            open(tmp_filename, 'w').write("""
# comment
name=Peter
awesome=
# comment
[othersection]
foo=bar  # other comment
        """)

            try:
                o = for_configobj.ValueSource(tmp_filename)
                r = {'othersection': {'foo': 'bar'},
                     'name': 'Peter',
                     'awesome': ''}
                assert o.get_values(None, None) == r
                # in the case of this implementation of a ValueSource,
                # the two parameters to get_values are dummies.  That may
                # not be true for all ValueSource implementations
                self.assertEqual(o.get_values(0, False), r)
                self.assertEqual(o.get_values(1, True), r)
                self.assertEqual(o.get_values(2, False), r)
                self.assertEqual(o.get_values(3, True), r)

                v = o.get_values(None, True, DotDict)
                self.assertTrue(isinstance(v, DotDict))
                v = o.get_values(None, None, obj_hook=DotDictWithAcquisition)
                self.assertTrue(isinstance(v, DotDictWithAcquisition))

            finally:
                if os.path.isfile(tmp_filename):
                    os.remove(tmp_filename)

        #----------------------------------------------------------------------
        def test_for_configobj_basics_2(self):
            tmp_filename = os.path.join(tempfile.gettempdir(), 'test.ini')
            open(tmp_filename, 'w').write("""
# comment
name=Peter
awesome=
# comment
[othersection]
foo=bar  # other comment
        """)

            try:
                o = for_configobj.ValueSource(tmp_filename)
                c = ConfigurationManager(
                    [],
                    use_admin_controls=True,
                    #use_config_files=False,
                    use_auto_help=False,
                    argv_source=[]
                )

                self.assertEqual(
                    o.get_values(c, False),
                    {
                        'othersection': {'foo': 'bar'},
                        'name': 'Peter',
                        'awesome': ''
                    }
                )
                self.assertEqual(
                    o.get_values(c, True),
                    {
                        'othersection': {'foo': 'bar'},
                        'name': 'Peter',
                        'awesome': ''
                    }
                )
            finally:
                if os.path.isfile(tmp_filename):
                    os.remove(tmp_filename)

        #----------------------------------------------------------------------
        # this test will be added back in the future
        def donttest_for_configobj_basics_3(self):
            n = Namespace()
            n.add_option("name", default='lars')
            n.add_option("awesome", default='lars')
            n.namespace('othersection')
            n.othersection.add_option('foo', default=23)
            tmp_filename = os.path.join(tempfile.gettempdir(), 'test.ini')
            open(tmp_filename, 'w').write("""
# comment
name=Peter
awesome=
# comment
[othersection]
bad_option=bar  # other comment
        """)

            try:

                self.assertRaises(
                    NotAnOptionError,
                    ConfigurationManager,
                    [n],
                    [tmp_filename],
                )
            finally:
                if os.path.isfile(tmp_filename):
                    os.remove(tmp_filename)

        #----------------------------------------------------------------------
        def test_write_ini(self):
            n = self._some_namespaces()
            c = ConfigurationManager(
                [n],
                [{
                    "c.fred": "just like George Jetson",
                }],
                use_admin_controls=True,
                #use_config_files=False,
                use_auto_help=False,
                argv_source=[]
            )
            expected = """# the a
#aaa=2011-05-04T15:10:00

[c]

    # husband from Flintstones
    fred=just like George Jetson

    # wife from Flintstones
    #wilma=waspish's

[d]

    # female neighbor from I Love Lucy
    #ethel=silly

    # male neighbor from I Love Lucy
    #fred=crabby

[x]

    # the password
    #password=secret "message"

    # how big in tons
    #size=100
"""
            out = StringIO()
            c.write_conf(for_configobj, opener=stringIO_context_wrapper(out))
            received = out.getvalue()
            out.close()
            self.assertEqual(expected.strip(), received.strip())

        #----------------------------------------------------------------------
        def test_write_ini_with_reference_value_froms(
            self
        ):
            n = self._some_namespaces()
            n.namespace('x1')
            n.x1.add_option(
                'password',
                default='secret "message"',
                doc='the password',
                likely_to_be_changed=True,
                reference_value_from='xxx.yyy'
            )
            n.namespace('x2')
            n.x2.add_option(
                'password',
                default='secret "message"',
                doc='the password',
                reference_value_from='xxx.yyy'
            )
            external_values = {
                'xxx': {
                    'yyy': {
                        'password': 'dwight and wilma'
                    }
                },
            }
            c = ConfigurationManager(
                [n],
                values_source_list=[external_values],
                use_admin_controls=True,
                use_auto_help=False,
                argv_source=[]
            )
            expected = ("""# the a
#aaa=2011-05-04T15:10:00

[xxx]

    #+include ./common_xxx.ini

    [[yyy]]

        #+include ./common_yyy.ini

        # the password
        password=dwight and wilma

[c]

    # husband from Flintstones
    #fred="stupid, deadly"

    # wife from Flintstones
    #wilma=waspish's

[d]

    # female neighbor from I Love Lucy
    #ethel=silly

    # male neighbor from I Love Lucy
    #fred=crabby

[x]

    # the password
    #password=secret "message"

    # how big in tons
    #size=100

[x1]

    # the password
    # see "xxx.yyy.password" for the default or override it here
    password=dwight and wilma

[x2]

    # the password
    # see "xxx.yyy.password" for the default or override it here
    password=dwight and wilma
""")
            out = StringIO()
            c.write_conf(for_configobj, opener=stringIO_context_wrapper(out))
            received = out.getvalue()
            out.close()
            self.assertEqual(expected.strip(), received.strip())

        #----------------------------------------------------------------------
        def test_write_ini_with_custom_converters(self):

            def dict_encoder(dict_):
                return ','.join('%s:%s' % (k, v) for (k, v) in dict_.items())

            def dict_decoder(string):
                return dict(x.split(':') for x in string.split(','))

            n = Namespace(doc='top')
            n.add_option(
                'a',
                default={'one': 'One'},
                doc='the doc string',
                to_string_converter=dict_encoder,
                from_string_converter=dict_decoder,
            )
            c = ConfigurationManager(
                [n],
                use_admin_controls=True,
                use_auto_help=False,
                argv_source=[]
            )
            expected = "# the doc string\n#a=one:One\n"
            out = StringIO()
            c.write_conf(for_configobj, opener=stringIO_context_wrapper(out))
            received = out.getvalue()
            out.close()

            self.assertEqual(expected.strip(), received.strip())

        #----------------------------------------------------------------------
        def test_configobj_includes_inside_sections(self):
            include_file_name = ''
            ini_file_name = ''
            try:
                with tempfile.NamedTemporaryFile(
                    'w',
                    suffix='ini',
                    delete=False
                ) as f:
                    include_file_name = f.name
                    contents = (
                        'dbhostname=myserver\n'
                        'dbname=some_database\n'
                        'dbuser=dwight\n'
                        'dbpassword=secrets\n'
                    )
                    f.write(contents)

                with tempfile.NamedTemporaryFile(
                    'w',
                    suffix='ini',
                    delete=False
                ) as f:
                    ini_file_name = f.name
                    contents = (
                        '[source]\n'
                        '+include %s\n'
                        '\n'
                        '[destination]\n'
                        '+include %s\n'
                        % (include_file_name, include_file_name)
                    )
                    f.write(contents)
                o = for_configobj.ValueSource(ini_file_name)
                expected_dict = {
                    'source': {
                        'dbhostname': 'myserver',
                        'dbname': 'some_database',
                        'dbuser': 'dwight',
                        'dbpassword': 'secrets'
                    },
                    'destination': {
                        'dbhostname': 'myserver',
                        'dbname': 'some_database',
                        'dbuser': 'dwight',
                        'dbpassword': 'secrets'
                    }
                }
                self.assertEqual(o.get_values(1, True), expected_dict)
            finally:
                if os.path.isfile(include_file_name):
                    os.remove(include_file_name)
                if os.path.isfile(ini_file_name):
                    os.remove(ini_file_name)

        #----------------------------------------------------------------------
        def test_configobj_includes_outside_a_section(self):
            include_file_name = ''
            ini_file_name = ''
            try:
                with tempfile.NamedTemporaryFile(
                    'w',
                    suffix='ini',
                    delete=False
                ) as f:
                    include_file_name = f.name
                    contents = (
                        'dbhostname=myserver\n'
                        'dbname=some_database\n'
                        'dbuser=dwight\n'
                        'dbpassword=secrets\n'
                    )
                    f.write(contents)

                with tempfile.NamedTemporaryFile(
                    'w',
                    suffix='ini',
                    delete=False
                ) as f:
                    ini_file_name = f.name
                    contents = (
                        '+include %s\n'
                        '\n'
                        '[destination]\n'
                        'x = y\n'
                        'foo=bar'
                        % include_file_name
                    )
                    f.write(contents)
                o = for_configobj.ValueSource(ini_file_name)
                expected_dict = {
                    'dbhostname': 'myserver',
                    'dbname': 'some_database',
                    'dbuser': 'dwight',
                    'dbpassword': 'secrets',
                    'destination': {
                        'x': 'y',
                        'foo': 'bar'
                    }
                }
                self.assertEqual(o.get_values(1, True), expected_dict)
            finally:
                if os.path.isfile(include_file_name):
                    os.remove(include_file_name)
                if os.path.isfile(ini_file_name):
                    os.remove(ini_file_name)

        #----------------------------------------------------------------------
        def test_configobj_relative_includes(self):
            include_file_name = ''
            ini_file_name = ''
            try:
                db_creds_dir = tempfile.mkdtemp()
                db_creds_basename = os.path.basename(db_creds_dir)
                ini_repo_dir = tempfile.mkdtemp()
                with tempfile.NamedTemporaryFile(
                    'w',
                    suffix='ini',
                    dir=db_creds_dir,
                    delete=False
                ) as f:
                    include_file_name = f.name
                    include_file_basename = os.path.basename(f.name)
                    contents = (
                        'dbhostname=myserver\n'
                        'dbname=some_database\n'
                        'dbuser=dwight\n'
                        'dbpassword=secrets\n'
                    )
                    f.write(contents)

                with tempfile.NamedTemporaryFile(
                    'w',
                    suffix='ini',
                    dir=ini_repo_dir,
                    delete=False
                ) as f:
                    ini_file_name = f.name
                    contents = (
                        '+include ../%s/%s\n'
                        '\n'
                        '[destination]\n'
                        '+include ../%s/%s\n'
                        % (
                            db_creds_basename,
                            include_file_basename,
                            db_creds_basename,
                            include_file_basename
                        )
                    )
                    f.write(contents)
                o = for_configobj.ValueSource(ini_file_name)
                expected_dict = {
                    'dbhostname': 'myserver',
                    'dbname': 'some_database',
                    'dbuser': 'dwight',
                    'dbpassword': 'secrets',
                    'destination': {
                        'dbhostname': 'myserver',
                        'dbname': 'some_database',
                        'dbuser': 'dwight',
                        'dbpassword': 'secrets',
                        }
                }
                self.assertEqual(o.get_values(1, True), expected_dict)
            finally:
                if os.path.isfile(include_file_name):
                    os.remove(include_file_name)
                if os.path.isfile(ini_file_name):
                    os.remove(ini_file_name)
                if os.path.isdir(db_creds_dir):
                    os.rmdir(db_creds_dir)
                if os.path.isdir(ini_repo_dir):
                    os.rmdir(ini_repo_dir)

        #----------------------------------------------------------------------
        def test_configobj_relative_paths(self):
            include_file_name = ''
            ini_file_name = ''
            try:
                db_creds_dir = tempfile.mkdtemp()
                ini_repo_dir = tempfile.mkdtemp()
                with tempfile.NamedTemporaryFile(
                    'w',
                    suffix='ini',
                    dir=db_creds_dir,
                    delete=False
                ) as f:
                    include_file_name = f.name
                    include_file_basename = os.path.basename(f.name)
                    contents = (
                        'dbhostname=myserver\n'
                        'dbname=some_database\n'
                        'dbuser=dwight\n'
                        'dbpassword=secrets\n'
                    )
                    f.write(contents)
                with tempfile.NamedTemporaryFile(
                    'w',
                    suffix='ini',
                    dir=db_creds_dir,
                    delete=False
                ) as f:
                    ini_file_name = f.name
                    contents = (
                        '+include %s\n'
                        % include_file_basename
                    )
                    f.write(contents)
                o = for_configobj.ValueSource(ini_file_name)
                expected_dict = {
                    'dbhostname': 'myserver',
                    'dbname': 'some_database',
                    'dbuser': 'dwight',
                    'dbpassword': 'secrets',
                }
                self.assertEqual(o.get_values(1, True), expected_dict)
            finally:
                if os.path.isfile(include_file_name):
                    os.remove(include_file_name)
                if os.path.isfile(ini_file_name):
                    os.remove(ini_file_name)
                if os.path.isdir(db_creds_dir):
                    os.rmdir(db_creds_dir)
                if os.path.isdir(ini_repo_dir):
                    os.rmdir(ini_repo_dir)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest

from configman.memoize import memoize

#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def test_memoize_function(self):

        @memoize()
        def foo(a, b, c):
            foo.counter += 1
            return (a, b, c)
        foo.counter = 0

        for i in range(5):
            # repeatable and exactly 10 function calls
            results = [foo(x, x, x) for x in range(10)]
            expected = [(x, x, x) for x in range(10)]
            self.assertEqual(results, expected)
            self.assertEqual(foo.counter, 10)

    #--------------------------------------------------------------------------
    def test_memoize_instance_method(self):

        class A(object):
            def __init__(self):
                self.counter = 0
            @memoize()
            def foo(self, a, b, c):
                self.counter += 1
                return (a, b, c)

        a = A()
        for i in range(5):
            # repeatable and exactly 10 function calls
            results = [a.foo(x, x, x) for x in range(10)]
            expected = [(x, x, x) for x in range(10)]
            self.assertEqual(results, expected)
            self.assertEqual(a.counter, 10)

        b = A()
        # repeatable and exactly 10 function calls
        results = [b.foo(x, x, x) for x in range(10)]
        expected = [(x, x, x) for x in range(10)]
        self.assertEqual(results, expected)
        self.assertEqual(b.counter, 10)

    #--------------------------------------------------------------------------
    def test_memoize_class_method(self):

        class A(object):
            counter = 0

            @classmethod
            @memoize()
            def foo(klass, a, b, c):
                klass.counter += 1
                return (a, b, c)

        for i in range(5):
            # repeatable and exactly 10 function calls
            results = [A.foo(x, x, x) for x in range(10)]
            expected = [(x, x, x) for x in range(10)]
            self.assertEqual(results, expected)
            self.assertEqual(A.counter, 10)

    #--------------------------------------------------------------------------
    def test_memoize_static_method(self):

        class A(object):
            counter = 0

            @staticmethod
            @memoize()
            def foo(a, b, c):
                A.counter += 1
                return (a, b, c)

        for i in range(5):
            # repeatable and exactly 10 function calls
            results = [A.foo(x, x, x) for x in range(10)]
            expected = [(x, x, x) for x in range(10)]
            self.assertEqual(results, expected)
            self.assertEqual(A.counter, 10)








# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import collections
import os
import sys

from configman.value_sources.source_exceptions import CantHandleTypeException
from configman.option import Option
from configman.dotdict import DotDict
from configman.memoize import memoize
from configman import namespace

can_handle = (
    os.environ,
    collections.Mapping,
)

file_name_extension = 'env'


#==============================================================================
class ValueSource(object):
    #--------------------------------------------------------------------------
    def __init__(self, source, the_config_manager=None):
        if source is os.environ:
            self.always_ignore_mismatches = True
        elif isinstance(source, collections.Mapping):
            if "always_ignore_mismatches" in source:
                self.always_ignore_mismatches = \
                    bool(source["always_ignore_mismatches"])
            else:
                self.always_ignore_mismatches = False
        else:
            raise CantHandleTypeException()
        self.source = source

    #--------------------------------------------------------------------------
    @memoize()
    def get_values(self, config_manager, ignore_mismatches, obj_hook=DotDict):
        if isinstance(self.source, obj_hook):
            return self.source
        return obj_hook(initializer=self.source)

    #--------------------------------------------------------------------------
    @staticmethod
    def write(source_mapping, output_stream=sys.stdout):
        ValueSource._write_ini(source_mapping, output_stream=output_stream)

    #--------------------------------------------------------------------------
    @staticmethod
    def _namespace_reference_value_from_sort(key_value_tuple):
        key, value = key_value_tuple
        if value._reference_value_from:
            # this forces referenced value sections to sort to the top.
            return 'aaaaaa' + key
        else:
            return key

    #--------------------------------------------------------------------------
    @staticmethod
    def write(source_dict, namespace_name=None, output_stream=sys.stdout):
        options = [
            value
            for value in source_dict.values()
            if isinstance(value, Option)
        ]
        options.sort(cmp=lambda x, y: cmp(x.name, y.name))
        namespaces = [
            (key, value)
            for key, value in source_dict.items()
            if isinstance(value, namespace.Namespace)
        ]

        def split_long_line(line, prefix='\n', max_length=80):
            parts = line.split()
            lines = []
            one = []
            for part in parts:
                if len(' '.join(one + [part])) > max_length - len(prefix):
                    lines.append(one)
                    one = []
                one.append(part)
            lines.append(one)
            lines.insert(0, '')
            return prefix.join([' '.join(x) for x in lines])

        for an_option in options:
            if namespace_name:
                option_name = "%s.%s" % (namespace_name, an_option.name)
            else:
                option_name = an_option.name
            option_value = str(an_option)
            if isinstance(option_value, unicode):
                option_value = option_value.encode('utf8')

            comment_line = '%s (default: %r)' % (
                an_option.doc or '',
                an_option.default
            )
            comment_lines = split_long_line(comment_line, '\n# ').lstrip()
            print >>output_stream, comment_lines

            option_format = '%s=%r'
            print >>output_stream, option_format % (
              option_name.replace('.', '__'),
              option_value
            )
        for key, a_namespace in namespaces:
            if namespace_name:
                namespace_label = ''.join((namespace_name, '.', key))
            else:
                namespace_label = key
            print >> output_stream, ''
            ValueSource.write(
              a_namespace,
              namespace_name=namespace_label,
              output_stream=output_stream
            )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import getopt

import configman.config_manager as config_manager
from configman.config_exceptions import NotAnOptionError
from configman.value_sources.for_getopt import ValueSource
from configman.dotdict import DotDict, DotDictWithAcquisition


#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def test_for_getopt_basics(self):
        source = ['a', 'b', 'c']
        o = ValueSource(source)
        self.assertEqual(o.argv_source, source)

    #--------------------------------------------------------------------------
    def test_for_getopt_get_values(self):
        c = config_manager.ConfigurationManager(
          use_admin_controls=True,
          #use_config_files=False,
          use_auto_help=False,
          argv_source=[]
        )

        source = ['--limit', '10']
        o = ValueSource(source)
        self.assertEqual(o.get_values(c, True), {})
        self.assertRaises(NotAnOptionError,
                          o.get_values, c, False)

        c.option_definitions.add_option('limit', default=0)
        self.assertEqual(o.get_values(c, False), {'limit': '10'})
        self.assertEqual(o.get_values(c, True), {'limit': '10'})
        v = o.get_values(c, True, DotDict)
        self.assertTrue(isinstance(v, DotDict))
        v = o.get_values(c, True, DotDictWithAcquisition)
        self.assertTrue(isinstance(v, DotDictWithAcquisition))

    #--------------------------------------------------------------------------
    def test_for_getopt_get_values_with_short_form(self):
        c = config_manager.ConfigurationManager(
          use_admin_controls=True,
          #use_config_files=False,
          use_auto_help=False,
          argv_source=[]
        )

        source = ['-l', '10']
        o = ValueSource(source)
        c.option_definitions.add_option('limit', default=0, short_form='l')
        self.assertEqual(o.get_values(c, False), {'limit': '10'})
        self.assertEqual(o.get_values(c, True), {'limit': '10'})

    #--------------------------------------------------------------------------
    def test_for_getopt_get_values_with_aggregates(self):
        c = config_manager.ConfigurationManager(
            use_admin_controls=True,
            #use_config_files=False,
            use_auto_help=False,
            argv_source=[]
        )

        source = ['-l', '10']
        o = ValueSource(source)
        c.option_definitions.add_option('limit', default=0, short_form='l')
        c.option_definitions.add_aggregation('summer', lambda *x: False)
        self.assertEqual(o.get_values(c, False), {'limit': '10'})
        self.assertEqual(o.get_values(c, True), {'limit': '10'})

    #--------------------------------------------------------------------------
    def test_for_getopt_with_ignore(self):
        function = ValueSource.getopt_with_ignore
        args = ['a', 'b', 'c']
        o, a = function(args, '', [])
        self.assertEqual(o, [])
        self.assertEqual(a, args)
        args = ['-a', '14', '--fred', 'sally', 'ethel', 'dwight']
        o, a = function(args, '', [])
        self.assertEqual([], o)
        self.assertEqual(a, ['14', 'sally', 'ethel', 'dwight'])
        args = ['-a', '14', '--fred', 'sally', 'ethel', 'dwight']
        o, a = function(args, 'a:', [])
        self.assertEqual(o, [('-a', '14')])
        self.assertEqual(a, ['sally', 'ethel', 'dwight'])
        args = ['-a', '14', '--fred', 'sally', 'ethel', 'dwight']
        o, a = function(args, 'a', ['fred='])
        self.assertEqual(o, [('-a', ''), ('--fred', 'sally')])
        self.assertEqual(a, ['14', 'ethel', 'dwight'])

    #--------------------------------------------------------------------------
    def test_overlay_config_5(self):
        """test namespace definition w/getopt"""
        n = config_manager.Namespace()
        n.add_option('a', 1, doc='the a')
        n.b = 17
        n.add_option('c', False, doc='the c')
        c = config_manager.ConfigurationManager(
            [n],
            [['--a', '2', '--c']],
            use_admin_controls=True,
            use_auto_help=False,
            argv_source=[]
        )
        self.assertTrue(isinstance(c.option_definitions.b,
                                   config_manager.Option))
        self.assertEqual(c.option_definitions.a.value, 2)
        self.assertEqual(c.option_definitions.b.value, 17)
        self.assertEqual(c.option_definitions.b.default, 17)
        self.assertEqual(c.option_definitions.b.name, 'b')
        self.assertEqual(c.option_definitions.c.name, 'c')
        self.assertEqual(c.option_definitions.c.value, True)

    #--------------------------------------------------------------------------
    def test_overlay_config_6(self):
        """test namespace definition w/getopt"""
        n = config_manager.Namespace()
        n.add_option('a', doc='the a', default=1)
        n.b = 17
        n.c = config_manager.Namespace()
        n.c.add_option('extra', doc='the x', default=3.14159, short_form='e')
        c = config_manager.ConfigurationManager(
            [n],
            [['--a', '2', '--c.extra', '11.0']],
            use_admin_controls=True,
            use_auto_help=False
        )
        self.assertEqual(type(c.option_definitions.b), config_manager.Option)
        self.assertEqual(c.option_definitions.a.value, 2)
        self.assertEqual(c.option_definitions.b.value, 17)
        self.assertEqual(c.option_definitions.b.default, 17)
        self.assertEqual(c.option_definitions.b.name, 'b')
        self.assertEqual(c.option_definitions.c.extra.name, 'extra')
        self.assertEqual(c.option_definitions.c.extra.doc, 'the x')
        self.assertEqual(c.option_definitions.c.extra.default, '11.0')
        self.assertEqual(c.option_definitions.c.extra.value, 11.0)

    #--------------------------------------------------------------------------
    def test_overlay_config_6a(self):
        """test namespace w/getopt w/short form"""
        n = config_manager.Namespace()
        n.add_option('a', 1, doc='the a')
        n.b = 17
        n.c = config_manager.Namespace()
        n.c.add_option('extra', 3.14159, 'the x', short_form='e')
        c = config_manager.ConfigurationManager(
            [n],
            [getopt],
            use_admin_controls=True,
            argv_source=['--a', '2', '-e', '11.0'],
            use_auto_help=False
        )
        self.assertEqual(type(c.option_definitions.b), config_manager.Option)
        self.assertEqual(c.option_definitions.a.value, 2)
        self.assertEqual(c.option_definitions.b.value, 17)
        self.assertEqual(c.option_definitions.b.default, 17)
        self.assertEqual(c.option_definitions.b.name, 'b')
        self.assertEqual(c.option_definitions.c.extra.name, 'extra')
        self.assertEqual(c.option_definitions.c.extra.doc, 'the x')
        self.assertEqual(c.option_definitions.c.extra.default, '11.0')
        self.assertEqual(c.option_definitions.c.extra.value, 11.0)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from functools import wraps

#------------------------------------------------------------------------------
def memoize(max_cache_size=1000):
    """Python 2.4 compatible memoize decorator.
    It creates a cache that has a maximum size.  If the cache exceeds the max,
    it is thrown out and a new one made.  With such behavior, it is wise to set
    the cache just a little larger that the maximum expected need.

    Parameters:
      max_cache_size - the size to which a cache can grow
    """
    def wrapper(f):
        @wraps(f)
        def fn(*args, **kwargs):
            if kwargs:
                key = (args, tuple(kwargs.items()))
            else:
                key = args
            try:
                return fn.cache[key]
            except KeyError:
                if fn.count >= max_cache_size:
                    fn.cache = {}
                    fn.count = 0
                result = f(*args, **kwargs)
                fn.cache[key] = result
                fn.count += 1
                return result
            except TypeError, x:
                return f(*args, **kwargs)
        fn.cache = {}
        fn.count = 0
        return fn
    return wrapper





import collections

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# TODO: This is a temporary dispatch mechanism.  This whole system
# is to be changed to automatic discovery of the for_* modules

from configman.def_sources import for_mappings
from configman.def_sources import for_modules
from configman.def_sources import for_json

definition_dispatch = {
  collections.Mapping: for_mappings.setup_definitions,
  type(for_modules): for_modules.setup_definitions,
  str: for_json.setup_definitions,
  unicode: for_json.setup_definitions,
}


class UnknownDefinitionTypeException(Exception):
    pass


def setup_definitions(source, destination):
    target_setup_func = None
    try:
        target_setup_func = definition_dispatch[type(source)]
    except KeyError:
        for a_key in definition_dispatch.keys():
            if isinstance(source, a_key):
                target_setup_func = definition_dispatch[a_key]
                break
        if not target_setup_func:
            raise UnknownDefinitionTypeException(repr(type(source)))
    target_setup_func(source, destination)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import datetime


def datetime_from_ISO_string(s):
    """ Take an ISO date string of the form YYYY-MM-DDTHH:MM:SS.S
    and convert it into an instance of datetime.datetime
    """
    try:
        return datetime.datetime.strptime(s, '%Y-%m-%dT%H:%M:%S')
    except ValueError:
        try:
            return datetime.datetime.strptime(s, '%Y-%m-%d')
        except ValueError:
            return datetime.datetime.strptime(s, '%Y-%m-%dT%H:%M:%S.%f')


def date_from_ISO_string(s):
    """ Take an ISO date string of the form YYYY-MM-DD
    and convert it into an instance of datetime.date
    """
    return datetime.datetime.strptime(s, '%Y-%m-%d').date()


def datetime_to_ISO_string(aDate):
    """ Take a datetime and convert to string of the form YYYY-MM-DDTHH:MM:SS.S
    """
    return aDate.isoformat()


def date_to_ISO_string(aDate):
    """ Take a datetime and convert to string of the form YYYY-MM-DD
    """
    return aDate.strftime('%Y-%m-%d')


def hours_str_to_timedelta(hoursAsString):
    return datetime.timedelta(hours=int(hoursAsString))


def timedelta_to_seconds(td):
    return td.days * 24 * 60 * 60 + td.seconds


def str_to_timedelta(input_str):
    """ a string conversion function for timedelta for strings in the format
    DD:HH:MM:SS or D HH:MM:SS
    """
    try:
        input_str = input_str.replace(' ', ':')
    except (TypeError, AttributeError):
        from configman.converters import to_str
        raise TypeError('%s should have been a string' % to_str(input_str))
    days, hours, minutes, seconds = 0, 0, 0, 0
    details = input_str.split(':')
    if len(details) >= 4:
        days = int(details[-4])
    if len(details) >= 3:
        hours = int(details[-3])
    if len(details) >= 2:
        minutes = int(details[-2])
    if len(details) >= 1:
        seconds = int(details[-1])
    return datetime.timedelta(days=days,
                              hours=hours,
                              minutes=minutes,
                              seconds=seconds)


def timedelta_to_str(aTimedelta):
    """ a conversion function for time deltas to string in the form
    DD:HH:MM:SS
    """
    days = aTimedelta.days
    temp_seconds = aTimedelta.seconds
    hours = temp_seconds / 3600
    minutes = (temp_seconds - hours * 3600) / 60
    seconds = temp_seconds - hours * 3600 - minutes * 60
    return '%d %02d:%02d:%02d' % (days, hours, minutes, seconds)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from configman.def_sources.for_mappings import setup_definitions


def setup_definitions(source, destination):
    module_dict = source.__dict__.copy()
    del module_dict['__builtins__']
    setup_definitions(module_dict, destination)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest

from configman import (
    option, dotdict, namespace, ConfigurationManager, class_converter
)
from configman.def_sources import for_mappings


#==============================================================================
class MooseBase(object):
    required_config = {
        "nodename": "localhost",
        "url": "http://twobraids.com/",
        "auth_key": "675A8BFF2099",
    }

    #--------------------------------------------------------------------------
    def __init__(self, config):
        self.config = config

    #--------------------------------------------------------------------------
    def connect(self):
        return "MooseBase connection"  # actual connection stuff ommitted


#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def test_setup_definitions_1(self):
        s = dotdict.DotDict()
        s.x = option.Option('x', 17, 'the x')
        s.n = {'name': 'n', 'doc': 'the n', 'default': 23}
        s.__forbidden__ = option.Option(
            '__forbidden__',
            'no, you cannot',
            38
        )
        s.t = namespace.Namespace()
        s.t.add_option('kk', 999, 'the kk')
        s.w = 89
        s.z = None
        s.t2 = namespace.Namespace('empty namespace')
        d = dotdict.DotDict()
        for_mappings.setup_definitions(s, d)
        self.assertEqual(len(d), 6)
        self.assertTrue(isinstance(d.x, option.Option))
        self.assertTrue(isinstance(d.n, option.Option))
        self.assertTrue(d.n.name == 'n')
        self.assertTrue(d.n.default == 23)
        self.assertTrue(d.n.doc == 'the n')
        self.assertTrue(isinstance(d.t, namespace.Namespace))
        self.assertTrue(d.t.kk.name == 'kk')
        self.assertTrue(d.t.kk.default == 999)
        self.assertTrue(d.t.kk.doc == 'the kk')
        self.assertTrue(isinstance(d.w, namespace.Option))
        self.assertTrue(d.w.name == 'w')
        self.assertTrue(d.w.default == 89)
        self.assertTrue(d.w.doc == 'w')
        self.assertTrue(isinstance(d.t2, namespace.Namespace))
        self.assertTrue(len(d.t2) == 0)

    #--------------------------------------------------------------------------
    def test_setup_definitions_2(self):
        d = {'cls': MooseBase}

        cm = ConfigurationManager(d, values_source_list=[])
        c = cm.get_config()
        self.assertTrue(
            cm.option_definitions.cls.from_string_converter is class_converter
        )
        self.assertTrue(c.cls is MooseBase)




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import contextlib
import re

from cStringIO import StringIO
from datetime import datetime, timedelta, date

from mock import Mock

from configman import (
    RequiredConfig,
    Namespace,
    ConfigurationManager,
    command_line
)
from configman.option import Option
from configman.config_exceptions import CannotConvertError, NotAnOptionError
from configman.value_sources.for_modules import ValueSource
from configman.converters import class_converter


#==============================================================================
class Alpha(RequiredConfig):
    required_config = Namespace()
    required_config.add_option('a', doc='a', default=17)

    #--------------------------------------------------------------------------
    def __init__(self, config):
        self.config = config
        self.a = config.a


#==============================================================================
class Beta(RequiredConfig):
    required_config = Namespace()
    required_config.add_option(
        'b',
        doc='b',
        default=23
    )

    #--------------------------------------------------------------------------
    def __init__(self, config):
        self.config = config
        self.b = config.b

#==============================================================================
class Delta(RequiredConfig):
    required_config = Namespace()
    required_config.add_option(
        'messy',
        doc='messy',
        default=-99
    )
    required_config.add_option(
        'dd',
        doc='dd',
        default=Beta,
        from_string_converter=class_converter
    )

#==========================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def test_basic_import(self):
        config_manager = Mock()
        vs = ValueSource('configman.tests.values_for_module_tests_1')
        v = vs.get_values(config_manager, True)

        self.assertEqual(v['a'], 18)
        self.assertEqual(v['b'], [1, 2, 3, 3])
        self.assertEqual(v['c'], set(v['b']))
        self.assertEqual(v['d']['a'], v['a'])
        self.assertEqual(v['d']['b'], v['b'])
        self.assertEqual(v['d']['c'], v['c'])
        self.assertEqual(v['d']['d'], {1: 'one', 2: 'two'})
        self.assertEqual(v['foo'](1, 2, 3), '123')
        self.assertTrue('partial' not in v)
        self.assertEqual(v['bar'](b=8, c=9), '1889')
        self.assertEqual(str(v['Alpha'](*list(v['c']))), '123')

        self.assertFalse('x' in v)
        self.assertFalse('y' in v)
        self.assertFalse('os' in v)
        self.assertTrue('collections' in v)

        self.assertTrue('__package__' not in v.keys())
        self.assertTrue('__builtins__' not in v.keys())
        self.assertTrue('__doc__' in v.keys())
        self.assertTrue(v.__doc__.startswith('This is a test'))

        from collections import Mapping
        self.assertTrue(v.collections.Mapping is Mapping)
        from types import ModuleType
        self.assertTrue(isinstance(v.collections, ModuleType))

    #--------------------------------------------------------------------------
    def test_failure_1(self):
        """complete failure, the module does not exist"""
        self.assertRaises(
            CannotConvertError,
            ValueSource,
            'configman.tests.values_4_module_tests_1'
        )

    #--------------------------------------------------------------------------
    def test_failure_2(self):
        """wrong format, don't use os style paths, use dotted forms"""
        self.assertRaises(
            CannotConvertError,
            ValueSource,
            'configman/tests/test_val_for_modules.py'
        )

    #--------------------------------------------------------------------------
    def test_as_overlay(self):
        rc = Namespace()
        rc.add_option(
            'a',
            default=23
        )
        rc.add_option(
            'b',
            default='this is b'
        )
        rc.namespace('n')
        rc.n.add_option(
            'x',
            default=datetime(1999, 12, 31, 11, 59)
        )
        rc.n.add_option(
            'y',
            default=timedelta(3)
        )
        rc.n.add_option(
            'z',
            default=date(1650, 10, 2)
        )
        rc.dynamic_load = None

        cm = ConfigurationManager(
            [rc],
            values_source_list=[
                'configman.tests.values_for_module_tests_2',
                'configman.tests.values_for_module_tests_3',
            ]
        )
        config = cm.get_config()

        self.assertEqual(config.a, 99)
        self.assertEqual(config.b, 'now is the time')
        self.assertEqual(config.n.x,  datetime(1960, 5, 4, 15, 10))
        self.assertEqual(config.n.y,  timedelta(1))
        self.assertEqual(config.n.z, date(1960, 5, 4))
        from configman.tests.values_for_module_tests_3 import Alpha
        self.assertEqual(config.dynamic_load, Alpha)
        self.assertEqual(config.host, 'localhost')
        self.assertEqual(config.port, 5432)

    #--------------------------------------------------------------------------
    def test_as_overlay_bad_symbols_with_strict(self):
        rc = Namespace()
        rc.add_option(
            'a',
            default=23
        )
        rc.add_option(
            'b',
            default='this is b'
        )
        rc.namespace('n')
        rc.n.add_option(
            'x',
            default=datetime(1999, 12, 31, 11, 59)
        )
        rc.n.add_option(
            'y',
            default=timedelta(3)
        )
        rc.n.add_option(
            'z',
            default=date(1650, 10, 2)
        )
        rc.dynamic_load = None

        self.assertRaises(
            NotAnOptionError,
            ConfigurationManager,
            [rc],
            values_source_list=[
                'configman.tests.values_for_module_tests_2',
                'configman.tests.values_for_module_tests_4',
                command_line
            ],
            argv_source=['--admin.strict'],
        )

    #--------------------------------------------------------------------------
    def test_write_simple(self):
        rc = Namespace()
        rc.add_option(
            'a',
            default=23
        )
        rc.add_option(
            'b',
            default='this is b'
        )
        rc.namespace('n')
        rc.n.add_option(
            'x',
            default=datetime(1999, 12, 31, 11, 59)
        )
        rc.n.add_option(
            'y',
            default=timedelta(3)
        )
        rc.n.add_option(
            'z',
            default=date(1650, 10, 2)
        )

        cm = ConfigurationManager(
            [rc],
            values_source_list=[
                {
                    'a': 68,
                    'n.x': datetime(1960, 5, 4, 15, 10),
                    'n.y': timedelta(3),
                    'n.z': date(2001, 1, 1)
                }
            ]
        )
        s = StringIO()

        @contextlib.contextmanager
        def s_opener():
            yield s

        cm.write_conf('py', s_opener)
        r = s.getvalue()
        g = {}
        l = {}
        exec r in g, l
        self.assertEqual(l['a'], 68)
        self.assertEqual(l['b'], 'this is b')
        self.assertEqual(l['n'].x, datetime(1960, 5, 4, 15, 10))
        self.assertEqual(l['n'].y, timedelta(3))
        self.assertEqual(l['n'].z, date(2001, 1, 1))

    #--------------------------------------------------------------------------
    def test_write_with_imported_module(self):
        import os
        from configman.tests.values_for_module_tests_1 import Alpha, foo, a

        definitions = {
            'os_module': os,
            'a': 17,
            'imported_class': Alpha,
            'imported_function': foo,
            'xxx': {
                'yyy': a,
            }
        }
        cm = ConfigurationManager(
            definitions,
            values_source_list=[],
        )
        s = StringIO()

        @contextlib.contextmanager
        def s_opener():
            yield s

        cm.write_conf('py', s_opener)
        generated_python_module_text = s.getvalue()
        expected = """# generated Python configman file

from configman.dotdict import DotDict
from configman.tests.values_for_module_tests_1 import (
    foo,
    Alpha,
)

import os

# the following symbols will be ignored by configman when
# this module is used as a value source.  This will
# suppress the mismatch warning since these symbols are
# values for options, not option names themselves.
ignore_symbol_list = [
    "Alpha",
    "DotDict",
    "foo",
    "os",
]


# a
a = 17

# imported_class
imported_class = Alpha

# imported_function
imported_function = foo

# os_module
os_module = os

# Namespace: xxx
xxx = DotDict()

# yyy
xxx.yyy = 18
"""
        self.assertEqual(generated_python_module_text, expected)

    #--------------------------------------------------------------------------
    def test_write_with_imported_module_with_internal_mappings(self):
        import os
        from configman.tests.values_for_module_tests_1 import Alpha, foo

        d = {
            'a': 18,
            'b': 'hello',
            'c': [1, 2, 3],
            'd': {
                'host': 'localhost',
                'port': 5432,
            }
        }

        definitions = {
            'os_module': os,
            'a': 17,
            'imported_class': Alpha,
            'imported_function': foo,
            'xxx': {
                'yyy': Option('yyy', default=d)
            },
            'e': None,
        }
        required_config = Namespace()
        required_config.add_option(
          'minimal_version_for_understanding_refusal',
          doc='ignore the Thottleable protocol',
          default={'Firefox': '3.5.4'},
        )

        cm = ConfigurationManager(
            [definitions, required_config],
            values_source_list=[],
        )

        config = cm.get_config()

        s = StringIO()

        @contextlib.contextmanager
        def s_opener():
            yield s

        cm.write_conf('py', s_opener)
        generated_python_module_text = s.getvalue()

        expected = """# generated Python configman file

from configman.dotdict import DotDict
from configman.tests.values_for_module_tests_1 import (
    foo,
    Alpha,
)

import os

# the following symbols will be ignored by configman when
# this module is used as a value source.  This will
# suppress the mismatch warning since these symbols are
# values for options, not option names themselves.
ignore_symbol_list = [
    "Alpha",
    "DotDict",
    "foo",
    "os",
]


# a
a = 17

# e
e = None

# imported_class
imported_class = Alpha

# imported_function
imported_function = foo

# ignore the Thottleable protocol
minimal_version_for_understanding_refusal = {
    "Firefox": "3.5.4"
}

# os_module
os_module = os

# Namespace: xxx
xxx = DotDict()

xxx.yyy = {
    "a": 18,
    "b": "hello",
    "c": [
        1,
        2,
        3
    ],
    "d": {
        "host": "localhost",
        "port": 5432
    }
}
"""
        self.assertEqual(generated_python_module_text, expected)

    #--------------------------------------------------------------------------
    def test_write_with_imported_module_with_regex(self):
        required_config = Namespace()
        required_config.add_option(
          'identifier',
          doc='just an identifier re',
          default=r'[a-zA-Z][a-zA-Z0-9]*',
          from_string_converter=re.compile
        )
        cm = ConfigurationManager(
            required_config,
            values_source_list=[],
        )
        config = cm.get_config()

        s = StringIO()

        @contextlib.contextmanager
        def s_opener():
            yield s

        cm.write_conf('py', s_opener)
        generated_python_module_text = s.getvalue()
        expected = """# generated Python configman file



# just an identifier re
identifier = "[a-zA-Z][a-zA-Z0-9]*"
"""
        self.assertEqual(generated_python_module_text, expected)


    #--------------------------------------------------------------------------
    def test_write_skip_aggregations(self):
        required_config = Namespace()
        required_config.add_option(
          'minimal_version_for_understanding_refusal',
          doc='ignore the Thottleable protocol',
          default={'Firefox': '3.5.4'},
        )
        required_config.add_aggregation(
            'an_agg',
            lambda x, y, z: 'I refuse'
        )

        cm = ConfigurationManager(
            required_config,
            values_source_list=[],
        )

        config = cm.get_config()

        s = StringIO()

        @contextlib.contextmanager
        def s_opener():
            yield s

        cm.write_conf('py', s_opener)
        generated_python_module_text = s.getvalue()

        expected = """# generated Python configman file



# ignore the Thottleable protocol
minimal_version_for_understanding_refusal = {
    "Firefox": "3.5.4"
}
"""
        self.assertEqual(generated_python_module_text, expected)




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import os
import contextlib


from configman.datetime_util import (
    datetime_from_ISO_string
)
from configman.namespace import Namespace
from configman.config_manager import ConfigurationManager
from configman.config_file_future_proxy import ConfigFileFutureProxy

from configman import command_line


#--------------------------------------------------------------------------
def stringIO_context_wrapper(a_stringIO_instance):
    @contextlib.contextmanager
    def stringIS_context_manager():
        yield a_stringIO_instance
    return stringIS_context_manager


#==========================================================================
class TestCase(unittest.TestCase):
    def _some_namespaces(self):
        """set up some namespaces"""
        n = Namespace(doc='top')
        n.add_option(
            'aaa',
            '2011-05-04T15:10:00',
            'the a',
            short_form='a',
            from_string_converter=datetime_from_ISO_string
        )
        n.c = Namespace(doc='c space')
        n.c.add_option(
            'dwight',
            'stupid, deadly',
            'husband from Flintstones'
        )
        n.c.add_option('wilma', "waspish's", 'wife from Flintstones')
        n.d = Namespace(doc='d space')
        n.d.add_option('dwight', "crabby", 'male neighbor from I Love Lucy')
        n.d.add_option(
            'ethel',
            'silly',
            'female neighbor from I Love Lucy'
        )
        n.x = Namespace(doc='x space')
        n.x.add_option('size', 100, 'how big in tons', short_form='s')
        n.x.add_option('password', 'secret "message"', 'the password')
        return n

    #----------------------------------------------------------------------
    def test_no_config_and_not_required(self):
        ConfigurationManager(
            definition_source=self._some_namespaces(),
            values_source_list=[ConfigFileFutureProxy],
            app_name='dwight',
            config_pathname='.',
            config_optional=True,
        )

    #----------------------------------------------------------------------
    def test_no_config_and_config_required(self):
        self.assertRaises(
            IOError,
            ConfigurationManager,
            definition_source=self._some_namespaces(),
            values_source_list=[ConfigFileFutureProxy],
            app_name='dwight',
            config_pathname='.',
            config_optional=False,
        )

    #----------------------------------------------------------------------
    def test_configdir_exists_but_no_config_and_not_required(self):
        temp_config_dir = '/tmp'
        ConfigurationManager(
            definition_source=self._some_namespaces(),
            values_source_list=[ConfigFileFutureProxy],
            app_name='dwight',
            config_pathname=temp_config_dir,
            config_optional=True,
        )

    #----------------------------------------------------------------------
    def test_configdir_exists_but_no_config_and_config_required(self):
        temp_config_dir = '/tmp'
        self.assertRaises(
            IOError,
            ConfigurationManager,
            definition_source=self._some_namespaces(),
            values_source_list=[ConfigFileFutureProxy],
            app_name='dwight',
            config_pathname=temp_config_dir,
            config_optional=False,
        )

    #----------------------------------------------------------------------
    def test_configdir_and_default_config_exists_and_config_not_required(self):
        temp_config_dir = '/tmp'
        with open('/tmp/dwight.ini', 'w') as f:
            f.write('')
        try:
            ConfigurationManager(
                definition_source=self._some_namespaces(),
                values_source_list=[ConfigFileFutureProxy],
                app_name='dwight',
                config_pathname=temp_config_dir,
                config_optional=True,
            )
        finally:
            os.unlink('/tmp/dwight.ini')

    #----------------------------------------------------------------------
    def test_configdir_exists_and_default_config_exists_and_config_required(
        self
    ):
        temp_config_dir = '/tmp'
        with open('/tmp/dwight.ini', 'w') as f:
            f.write('[x]\n    size=42')
        try:
            cm = ConfigurationManager(
                definition_source=self._some_namespaces(),
                values_source_list=[ConfigFileFutureProxy],
                app_name='dwight',
                config_pathname=temp_config_dir,
                config_optional=False,
            )
            self.assertEqual(cm.get_config()['x.size'], 42)
        finally:
            os.unlink('/tmp/dwight.ini')

    #----------------------------------------------------------------------
    def test_overridden_configfile_doesnt_exist(
        self
    ):
        temp_config_dir = '/tmp'
        with open('/tmp/dwight.ini', 'w') as f:
            f.write('')
        try:
            self.assertRaises(
                IOError,
                ConfigurationManager,
                definition_source=self._some_namespaces(),
                values_source_list=[
                    ConfigFileFutureProxy,
                    command_line
                ],
                argv_source=['--admin.conf=wilma.ini'],
                app_name='dwight',
                config_pathname=temp_config_dir,
                config_optional=False,
            )
        finally:
            os.unlink('/tmp/dwight.ini')

    def test_default_config_exists_config_required_overridden_correctly(
        self
    ):
        temp_config_dir = '/tmp'
        with open('/tmp/dwight.ini', 'w') as f:
            f.write('[x]\n    size=42')
        try:
            with open('/tmp/wilma.ini', 'w') as f:
                f.write('[x]\n    size=666')
            try:
                cm = ConfigurationManager(
                    definition_source=self._some_namespaces(),
                    values_source_list=[
                        ConfigFileFutureProxy,
                        command_line
                    ],
                    argv_source=['--admin.conf=/tmp/wilma.ini'],
                    app_name='dwight',
                    config_pathname=temp_config_dir,
                    config_optional=False,
                )
                self.assertEqual(cm.get_config()['x.size'], 666)
            finally:
                os.unlink('/tmp/wilma.ini')
        finally:
            os.unlink('/tmp/dwight.ini')

    def test_configdir_default_config_config_required_overridden_correctly_2(
        self
    ):
        temp_config_dir = '/tmp'
        with open('/tmp/dwight.ini', 'w') as f:
            f.write('')
        try:
            with open('./wilma.ini', 'w') as f:
                f.write('')
            try:
                ConfigurationManager(
                    definition_source=self._some_namespaces(),
                    values_source_list=[
                        ConfigFileFutureProxy,
                        command_line
                    ],
                    argv_source=['--admin.conf=./wilma.ini'],
                    app_name='dwight',
                    config_pathname=temp_config_dir,
                    config_optional=False,
                )
            finally:
                os.unlink('./wilma.ini')
        finally:
            os.unlink('/tmp/dwight.ini')

    def test_configdir_default_config_config_required_overridden_incorrectly_2(
        self
    ):
        temp_config_dir = '/tmp'
        with open('/tmp/dwight.ini', 'w') as f:
            f.write('')
        try:
            with open('/tmp/wilma.ini', 'w') as f:
                f.write('')
            try:
                self.assertRaises(
                    IOError,
                    ConfigurationManager,
                    definition_source=self._some_namespaces(),
                    values_source_list=[
                        ConfigFileFutureProxy,
                        command_line
                    ],
                    argv_source=['--admin.conf=wilma.ini'],
                    app_name='dwight',
                    config_pathname=temp_config_dir,
                    config_optional=False,
                )
            finally:
                os.unlink('/tmp/wilma.ini')
        finally:
            os.unlink('/tmp/dwight.ini')



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import os
import json
import tempfile
import contextlib
from cStringIO import StringIO

from configman.namespace import Namespace
from configman.config_manager import ConfigurationManager
from configman.datetime_util import datetime_from_ISO_string
from configman.value_sources import for_json
from configman.value_sources.for_json import ValueSource
from configman.dotdict import DotDict, DotDictWithAcquisition


#------------------------------------------------------------------------------
def stringIO_context_wrapper(a_stringIO_instance):
    @contextlib.contextmanager
    def stringIO_context_manager():
        yield a_stringIO_instance
    return stringIO_context_manager


#------------------------------------------------------------------------------
def bbb_minus_one(config, local_config, args):
    return config.bbb - 1


#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def test_for_json_basics(self):
        tmp_filename = os.path.join(tempfile.gettempdir(), 'test.json')
        j = {
            'fred': 'wilma',
            'number': 23,
        }
        with open(tmp_filename, 'w') as f:
            json.dump(j, f)
        try:
            jvs = ValueSource(tmp_filename)
            vals = jvs.get_values(None, True)
            self.assertEqual(vals['fred'], 'wilma')
            self.assertEqual(vals['number'], 23)
        finally:
            if os.path.isfile(tmp_filename):
                os.remove(tmp_filename)

    #--------------------------------------------------------------------------
    def test_write_json(self):
        n = Namespace(doc='top')
        n.add_option(
            'aaa',
            '2011-05-04T15:10:00',
            'the a',
            short_form='a',
            from_string_converter=datetime_from_ISO_string
        )

        c = ConfigurationManager(
            [n],
            use_admin_controls=True,
            use_auto_help=False,
            argv_source=[]
        )

        out = StringIO()
        c.write_conf(for_json, opener=stringIO_context_wrapper(out))
        received = out.getvalue()
        out.close()
        jrec = json.loads(received)

        expect_to_find = {
            "short_form": "a",
            "default": "2011-05-04T15:10:00",
            "doc": "the a",
            "value": "2011-05-04T15:10:00",
            "from_string_converter":
            "configman.datetime_util.datetime_from_ISO_string",
            "name": "aaa"
        }
        for key, value in expect_to_find.items():
            self.assertEqual(jrec['aaa'][key], value)

    #--------------------------------------------------------------------------
    def test_json_round_trip(self):
        n = Namespace(doc='top')
        n.add_option(
            'aaa',
            '2011-05-04T15:10:00',
            'the a',
            short_form='a',
            from_string_converter=datetime_from_ISO_string
        )
        expected_date = datetime_from_ISO_string('2011-05-04T15:10:00')
        n.add_option(
            'bbb',
            '37',
            'the a',
            short_form='a',
            from_string_converter=int
        )
        n.add_option('write', 'json')
        n.add_aggregation('bbb_minus_one', bbb_minus_one)
        name = '/tmp/test.json'
        import functools
        opener = functools.partial(open, name, 'w')
        c1 = ConfigurationManager(
            [n],
            [],
            use_admin_controls=True,
            use_auto_help=False,
            app_name='/tmp/test',
            app_version='0',
            app_description='',
            argv_source=[]
        )
        c1.write_conf('json', opener)
        d1 = {'bbb': 88}
        d2 = {'bbb': '-99'}
        try:
            with open(name) as jfp:
                j = json.load(jfp)
            c2 = ConfigurationManager(
                (j,),
                (d1, d2),
                use_admin_controls=True,
                use_auto_help=False,
                argv_source=[]
            )
            config = c2.get_config()
            self.assertEqual(config.aaa, expected_date)
            self.assertEqual(config.bbb, -99)
            self.assertEqual(config.bbb_minus_one, -100)

        finally:
            os.unlink(name)

    def test_get_values(self):
        j = {
            'a': '1',
            'b': 2,
            'c': {
                'd': 'x',
                'e': 'y'
            },
            'd': {
                'd': 'X'
            }
        }
        tmp_filename = os.path.join(tempfile.gettempdir(), 'test.json')
        with open(tmp_filename, 'w') as f:
            json.dump(j, f)
        try:
            jvs = ValueSource(tmp_filename)
            vals = jvs.get_values(None, True, DotDict)
            self.assertTrue(isinstance(vals, DotDict))
            vals = jvs.get_values(None, True, DotDictWithAcquisition)
            self.assertTrue(isinstance(vals, DotDictWithAcquisition))
            self.assertEqual(vals.d.b, 2)
        finally:
            if os.path.isfile(tmp_filename):
                os.remove(tmp_filename)




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import collections

from configman.converters import (
    str_to_python_object,
    from_string_converters,
    to_str
)
from configman.config_exceptions import (
    CannotConvertError,
    OptionError
)


#==============================================================================
class Option(object):
    #--------------------------------------------------------------------------
    def __init__(
        self,
        name,
        default=None,
        doc=None,
        from_string_converter=None,
        to_string_converter=None,
        value=None,
        short_form=None,
        exclude_from_print_conf=False,
        exclude_from_dump_conf=False,
        is_argument=False,
        likely_to_be_changed=False,
        not_for_definition=False,
        reference_value_from=None,
        secret=False,
        has_changed=False,
        foreign_data=None,
    ):
        self.name = name
        self.short_form = short_form
        self.default = default
        if isinstance(doc, basestring):
            doc = doc.strip()
        self.doc = doc
        if from_string_converter is None:
            if default is not None:
                # take a qualified guess from the default value
                from_string_converter = self._deduce_converter(default)
        if isinstance(from_string_converter, basestring):
            from_string_converter = str_to_python_object(from_string_converter)
        self.from_string_converter = from_string_converter
        # if this is not set, the type is used in converters.py to attempt
        # the conversion
        self.to_string_converter = to_string_converter
        if value is None:
            value = default
        self.value = value
        self.is_argument = is_argument
        self.exclude_from_print_conf = exclude_from_print_conf
        self.exclude_from_dump_conf = exclude_from_dump_conf
        self.likely_to_be_changed = likely_to_be_changed
        self.not_for_definition = not_for_definition
        self.reference_value_from = reference_value_from
        self.secret = secret
        self.has_changed = has_changed
        self.foreign_data = foreign_data

    #--------------------------------------------------------------------------
    def __str__(self):
        """return an instance of Option's value as a string.

        The option instance doesn't actually have to be from the Option class.
        All it requires is that the passed option instance has a ``value``
        attribute.
        """
        try:
            return self.to_string_converter(self.value)
        except TypeError:
            return to_str(self.value)

    #--------------------------------------------------------------------------
    def __eq__(self, other):
        if isinstance(other, Option):
            return (
                self.name == other.name
                and
                self.default == other.default
                and
                self.doc == other.doc
                and
                self.short_form == other.short_form
                and
                self.value == other.value
            )

    #--------------------------------------------------------------------------
    def __repr__(self):  # pragma: no cover
        if self.default is None:
            return '<Option: %r>' % self.name
        else:
            return '<Option: %r, default=%r, value=%r, is_argument=%r>' % (
                self.name, self.default, self.value, self.is_argument
            )

    #--------------------------------------------------------------------------
    def _deduce_converter(self, default):
        default_type = type(default)

        return from_string_converters.get(
            default_type,
            default_type
        )

    #--------------------------------------------------------------------------
    def set_value(self, val=None):
        if val is None:
            val = self.default
        if isinstance(val, basestring):
            try:
                new_value = self.from_string_converter(val)
                self.has_changed = new_value != self.value
                self.value = new_value
            except TypeError:
                self.has_changed = val != self.value
                self.value = val
            except ValueError:
                error_message = "In '%s', '%s' fails to convert '%s'" % (
                    self.name,
                    self.from_string_converter,
                    val
                )
                raise CannotConvertError(error_message)
        elif isinstance(val, Option):
            self.has_changed = val.default != self.value
            self.value = val.default
        elif isinstance(val, collections.Mapping) and 'default' in val:
            self.set_value(val["default"])
        else:
            self.has_changed = val != self.value
            self.value = val

    #--------------------------------------------------------------------------
    def set_default(self, val, force=False):
        """this function allows a default to be set on an option that dosen't
        have one.  It is used when a base class defines an Option for use in
        derived classes but cannot predict what value would useful to the
        derived classes.  This gives the derived classes the opportunity to
        set a logical default appropriate for the derived class' context.

        For example:

            class A(RequiredConfig):
                required_config = Namespace()
                required_config.add_option(
                  'x',
                  default=None
                )

            class B(A):
                A.required_config.x.set_default(68)

        parameters:
            val - the value for the default
            force - normally this function only works on Options that have not
                    had a default set (default is None).  This boolean allows
                    you to override an existing default.
        """
        if self.default is None or force:
            self.default = val
            self.set_value(val)
            self.has_changed = True
        else:
            raise OptionError(
                "cannot override existing default without using the 'force' "
                "option"
            )

    #--------------------------------------------------------------------------
    def copy(self):
        """return a copy"""
        o = Option(
            name=self.name,
            default=self.default,
            doc=self.doc,
            from_string_converter=self.from_string_converter,
            to_string_converter=self.to_string_converter,
            value=self.value,
            short_form=self.short_form,
            exclude_from_print_conf=self.exclude_from_print_conf,
            exclude_from_dump_conf=self.exclude_from_dump_conf,
            is_argument=self.is_argument,
            likely_to_be_changed=self.likely_to_be_changed,
            not_for_definition=self.not_for_definition,
            reference_value_from=self.reference_value_from,
            secret=self.secret,
            has_changed=self.has_changed,
            foreign_data=self.foreign_data,
        )
        return o


#==============================================================================
class Aggregation(object):
    #--------------------------------------------------------------------------
    def __init__(
        self,
        name,
        function,
        secret=False,
    ):
        self.name = name
        if isinstance(function, basestring):
            self.function = str_to_python_object(function)
        else:
            self.function = function
        self.value = None
        self.secret = secret

    #--------------------------------------------------------------------------
    def aggregate(self, all_options, local_namespace, args):
        self.value = self.function(all_options, local_namespace, args)

    #--------------------------------------------------------------------------
    def __eq__(self, other):
        if isinstance(other, Aggregation):
            return (
                self.name == other.name
                and self.function == other.function
            )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import os

from cStringIO import StringIO
import contextlib
from configman import Namespace, ConfigurationManager
from configman.datetime_util import datetime_from_ISO_string

from configman.value_sources import for_mapping
from configman.value_sources.for_mapping import ValueSource
from configman.dotdict import DotDict, DotDictWithAcquisition

def stringIO_context_wrapper(a_stringIO_instance):
    @contextlib.contextmanager
    def stringIS_context_manager():
        yield a_stringIO_instance
    return stringIS_context_manager


#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def _some_namespaces(self):
        """set up some namespaces"""
        n = Namespace(doc='top')
        n.add_option(
            'aaa',
            '2011-05-04T15:10:00',
            'the a',
            short_form='a',
            from_string_converter=datetime_from_ISO_string
        )
        n.c = Namespace(doc='c space')
        n.c.add_option(
            'fred',
            'stupid',
            # deliberate whitespace to test that it gets stripped
            ' husband from Flintstones '
        )
        n.c.add_option('wilma', 'waspish', 'wife from Flintstones')
        n.c.e = Namespace(doc='e space')
        n.c.e.add_option(
            'dwight',
            default=97,
            doc='my uncle'
        )
        n.c.add_option(
            'dwight',
            default=98,
            doc='your uncle'
        )
        n.d = Namespace(doc='d space')
        n.d.add_option('fred', 'crabby', 'male neighbor from I Love Lucy')
        n.d.add_option(
            'ethel',
            'silly',
            'female neighbor from I Love Lucy'
        )
        n.x = Namespace(doc='x space')
        n.x.add_option('size', 100, 'how big in tons', short_form='s')
        n.x.add_option('password', 'secret', 'the password')
        return n

    def test_environ_ignores_mismatches(self):
        vs = ValueSource(os.environ)
        self.assertTrue(vs.always_ignore_mismatches)
        self.assertTrue(vs.source is os.environ)

    def test_mapping(self):
        m = {
            'a': '1',
            'b': 2
        }
        vs = ValueSource(m)
        self.assertFalse(vs.always_ignore_mismatches)
        self.assertTrue(vs.source is m)

        m = {
            'a': '1',
            'b': 2,
            'always_ignore_mismatches': False
        }
        vs = ValueSource(m)
        self.assertFalse(vs.always_ignore_mismatches)
        self.assertTrue(vs.source is m)

        m = {
            'a': '1',
            'b': 2,
            'always_ignore_mismatches': True
        }
        vs = ValueSource(m)
        self.assertTrue(vs.always_ignore_mismatches)
        self.assertTrue(vs.source is m)

    def test_get_values(self):
        m = {
            'a': '1',
            'b': 2,
            'c': {
                'd': 'x',
                'e': 'y'
            },
            'd': {
                'd': 'X'
            }
        }
        vs = ValueSource(m)
        v = vs.get_values(None, None)
        self.assertTrue(isinstance(v, DotDict))
        v = vs.get_values(None, None, obj_hook=DotDictWithAcquisition)
        self.assertTrue(isinstance(v, DotDictWithAcquisition))
        self.assertEqual(v.d.b, 2)

    #--------------------------------------------------------------------------
    def test_for_mapping_nested_namespaces(self):
        n = self._some_namespaces()
        cm = ConfigurationManager(
            n,
            values_source_list=[],
        )
        out = StringIO()
        cm.write_conf(for_mapping, opener=stringIO_context_wrapper(out))
        received = out.getvalue()
        out.close()
        expected = """
# the a (default: '2011-05-04T15:10:00')
aaa='2011-05-04T15:10:00'

# your uncle (default: 98)
c__dwight='98'
# husband from Flintstones (default: 'stupid')
c__fred='stupid'
# wife from Flintstones (default: 'waspish')
c__wilma='waspish'

# my uncle (default: 97)
c__e__dwight='97'

# female neighbor from I Love Lucy (default: 'silly')
d__ethel='silly'
# male neighbor from I Love Lucy (default: 'crabby')
d__fred='crabby'

# the password (default: 'secret')
x__password='secret'
# how big in tons (default: 100)
x__size='100'
        """.strip()
        self.assertEqual(received.strip(), expected)

    #--------------------------------------------------------------------------
    def test_for_mapping_long_doc_in_write_conf(self):
        n = self._some_namespaces()
        n = Namespace(doc='top')
        n.add_option(
            'aaa',
            'Default Value Goes In Here',
            'This time the documentation string is really long. So long '
            'that we have to write it on multiple lines.',
        )
        cm = ConfigurationManager(
            n,
            values_source_list=[],
        )
        out = StringIO()
        cm.write_conf(for_mapping, opener=stringIO_context_wrapper(out))
        received = out.getvalue()
        out.close()
        lines = received.splitlines()
        for line in received.splitlines():
            self.assertTrue(len(line) < 80, line)
        expected = """
# This time the documentation string is really long. So long that we have to
# write it on multiple lines. (default: 'Default Value Goes In Here')
aaa='Default Value Goes In Here'
        """.strip()
        self.assertEqual(received.strip(), expected)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from os import path
with open(path.join(path.dirname(__file__), 'version.txt')) as f:
    __version__ = f.read().strip()

# Having these here makes it possible to easily import once configman is
# installed.
# For example::
#
#    from configman import Namespace, ConfigurationManager
#

from configman.config_manager import ConfigurationManager
from configman.required_config import RequiredConfig
from configman.namespace import Namespace
from configman.config_file_future_proxy import ConfigFileFutureProxy
from configman.converters import (
    class_converter,
    regex_converter,
    timedelta_converter
)

from configman.environment import environment
# this next line brings in command_line and, if argparse is available,
# a definition of the configman version of ArgumentParser.  Why is it done
# with "import *" ? Because we don't know what symbols to import, the decision
# about what is symbols exist within the module.  To make the import specific
# here, it would be necessary to reproduce the same logic that is already
# in the commandline module.
from configman.commandline import *



#------------------------------------------------------------------------------
def configuration(*args, **kwargs):
    """this function just instantiates a ConfigurationManager and returns
    the configuration dictionary.  It accepts all the same parameters as the
    constructor for the ConfigurationManager class."""
    try:
        config_kwargs = {'mapping_class': kwargs.pop('mapping_class')}
    except KeyError:
        config_kwargs = {}
    cm = ConfigurationManager(*args, **kwargs)
    return cm.get_config(**config_kwargs)




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
import collections

from configman.namespace import Namespace

from configman.dotdict import DotDict
import configman.def_sources as defsrc


#==============================================================================
class TestCase(unittest.TestCase):

    #--------------------------------------------------------------------------
    def test_setup_definitions_1(self):
        d = DotDict()

        def fake_mapping_func(source, destination):
            self.assertTrue(isinstance(source, collections.Mapping))
            self.assertEqual(d, destination)
        saved_original = defsrc.definition_dispatch.copy()
        try:
            defsrc.definition_dispatch[collections.Mapping] = fake_mapping_func
            s = {}
            defsrc.setup_definitions(s, d)
            s = DotDict()
            defsrc.setup_definitions(s, d)
            s = Namespace()
            defsrc.setup_definitions(s, d)
        finally:
            defsrc.definition_dispatch = saved_original

    #--------------------------------------------------------------------------
    def test_setup_definitions_2(self):
        d = DotDict()

        def fake_mapping_func(source, destination):
            self.assertTrue(source is collections)
            self.assertEqual(d, destination)
        saved_original = defsrc.definition_dispatch.copy()
        try:
            defsrc.definition_dispatch[type(collections)] = fake_mapping_func
            s = collections
            defsrc.setup_definitions(s, d)
        finally:
            defsrc.definition_dispatch = saved_original

    #--------------------------------------------------------------------------
    def test_setup_definitions_3(self):
        d = DotDict()

        def fake_mapping_func(source, destination):
            self.assertTrue(isinstance(source, str))
            self.assertEqual(d, destination)
        saved_original = defsrc.definition_dispatch.copy()
        try:
            defsrc.definition_dispatch[str] = fake_mapping_func
            s = "{}"
            defsrc.setup_definitions(s, d)
        finally:
            defsrc.definition_dispatch = saved_original
